!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for bubble rise related arrays
!
!  This module also includes the following subroutines
!
!      allok8_bubble_rise
!      allok8_level_model
!      allok8_level_para
!
!  ===============================================================================================================================
!
      module bubble_rise
!
      use kind_specs
!
      integer(kind=ikd), save :: &
         num_bub, &  !  number of bubble rise sets
        siz_rbub, &  !  size of the array allocated for the real bub pointer arrays
        siz_ibub, &  !  size of the array allocated for the integer bub pointer arrays
        siz_lbub     !  size of the array allocated for the logical bub pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_bub(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_bub)
!  Space is allocated and pointers associations are made in subroutine allok8_bubble_rise
!
      real(kind=rkd), pointer, save :: &
           a1(:), &  !  partial density slope coefficient                
        a1old(:), &  !  previous time step value of a1                   
         alph(:), &  !  bubble gradient parameter                        
       alphin(:), &  !  input value of slope parameter
        alpho(:), &  !  previous time step slope parameter               
            b(:), &  !  partial density intercept coefficient            
         bold(:), &  !  previous time step value of b                    
         bubm(:), &  !  bubble mass                                      
        bubmo(:), &  !  previous time step bubble mass                   
        cdmgb(:), &  !  implicit solution damping coefficient            
       dvbdam(:), &  !  derivative of bubble vel wrt mix void frac
        dzmdt(:), &  !  time rate of change of mixture level             
       dzmodt(:), &  !  previous time step dzmdt                         
            e(:), &  !  mixture density slope coefficient                
         eold(:), &  !  previous time step value of e                    
            f(:), &  !  mixture density intercept coefficient            
         fold(:), &  !  previous time step value of f                    
         liqv(:), &  !  liquid volume                                    
        liqvo(:), &  !  previous time step liquid volume                 
         mixq(:)     !  mixture quality                                  
!
      real(kind=rkd), pointer, save :: &
         mixv(:), &  !  mixture volume                                   
        mixvo(:), &  !  previous time step mixture volume                
       vbnorm(:), &  !  bubble rise velocity normalization coefficient   
         vbub(:), &  !  bubble velocity                                  
       vbubin(:), &  !  input value of bubble rise velocity
        vbubo(:), &  !  previous time step bubble rise velocity          
           zm(:), &  !  mixture level                                    
          zmo(:)     !  previous time step mixture level                 
!
      integer(kind=ikd), target, allocatable, save :: i_bub(:)
      integer(kind=ikd), pointer, save :: &
       ibubol(:), &  !  volume number (old) for bubble rise calc         
        ibvol(:), &  !  volume number (new) for bubble rise calc         
       idalph(:), &  !  control system id for slope calc                 
       idvbub(:), &  !  control system id for velocity calc              
       iwilsn(:)     !  wilson bubble velocity flag                      
!
      logical(kind=lkd), target, allocatable, save :: l_bub(:)
      logical(kind=lkd), pointer, save :: &
       sepr8r(:)     !  logical flag indicating a separator              
!
!  ===============================================================================================================================
!  Specifications and space allocations for level model related arrays (stacked volumes)
!  ===============================================================================================================================
!
      integer(kind=ikd), save :: &
         num_lvl,  &  !  number of stack cards supplied
        siz_rlvl, &  !  size of the array allocated for the real level pointer arrays
        siz_ilvl     !  size of the array allocated for the integer level pointer arrays
!        
      real(kind=rkd), target, allocatable, save :: r_lvl(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_lvl)
!  Space is allocated and pointers associations are made in subroutine allok8_level
!
      real(kind=rkd), pointer, save :: &
       eqliql(:), &  !  equivalent liquid level for this stack
       eqmixl(:)     !  equivalent mixture level for this stack
!
      integer(kind=ikd), target, allocatable, save :: i_lvl(:)
      integer(kind=ikd), pointer, save :: &
       idxvsk(:), &  !  index to the first volume stack subfile
       nvolst(:)     !  number of volumes in stack
!       
!  ===============================================================================================================================
!  Specifications and space allocations for level parameter related arrays
!  subfile of the level model contains information for each volume in stack
!  ===============================================================================================================================
      integer(kind=ikd), save :: &
        num_stkvol, &  !  total number of stacked volumes
        siz_rlpara, &  !  size of the array allocated for the real level pointer arrays
        siz_ilpara     !  size of the array allocated for the integer level pointer arrays
!        
      real(kind=rkd), target, allocatable, save :: r_lpara(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:nvolst)
!  Space is allocated and pointers associations are made in subroutine allok8_lpara
!
      real(kind=rkd), pointer, save :: &
       botovl(:), &  !  volume overlap with the volume below
       topovl(:)     !  volume overlap with the volume above
!
      integer(kind=ikd), target, allocatable, save :: i_lpara(:)
      integer(kind=ikd), pointer, save :: &
       ivlidx(:)     !  volume index inside the stack
!       
      contains
!
!  ===============================================================================================================================
      subroutine allok8_bubble_rise
!
!  Subroutine to allocate space and set pointers for bub related arrays
!     Note: num_bub must be defined before allok8_bub is called
!
!     allok8_bubble_rise is called from subroutine inbubl
!
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_bub .le. 0) then
         write (6,110) num_bub
         call fail
         return
      endif
!
      if (allocated (r_bub)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_bub, i2,     a1, r_bub)
         call r_pntr (allocatd, num_bub, i2,  a1old, r_bub)
         call r_pntr (allocatd, num_bub, i2,   alph, r_bub)
         call r_pntr (allocatd, num_bub, i2, alphin, r_bub)
         call r_pntr (allocatd, num_bub, i2,  alpho, r_bub)
         call r_pntr (allocatd, num_bub, i2,      b, r_bub)
         call r_pntr (allocatd, num_bub, i2,   bold, r_bub)
         call r_pntr (allocatd, num_bub, i2,   bubm, r_bub)
         call r_pntr (allocatd, num_bub, i2,  bubmo, r_bub)
         call r_pntr (allocatd, num_bub, i2,  cdmgb, r_bub)
         call r_pntr (allocatd, num_bub, i2, dvbdam, r_bub)
         call r_pntr (allocatd, num_bub, i2,  dzmdt, r_bub)
         call r_pntr (allocatd, num_bub, i2, dzmodt, r_bub)
         call r_pntr (allocatd, num_bub, i2,      e, r_bub)
         call r_pntr (allocatd, num_bub, i2,   eold, r_bub)
         call r_pntr (allocatd, num_bub, i2,      f, r_bub)
         call r_pntr (allocatd, num_bub, i2,   fold, r_bub)
         call r_pntr (allocatd, num_bub, i2,   liqv, r_bub)
         call r_pntr (allocatd, num_bub, i2,  liqvo, r_bub)
         call r_pntr (allocatd, num_bub, i2,   mixq, r_bub)
         call r_pntr (allocatd, num_bub, i2,   mixv, r_bub)
         call r_pntr (allocatd, num_bub, i2,  mixvo, r_bub)
         call r_pntr (allocatd, num_bub, i2, vbnorm, r_bub)
         call r_pntr (allocatd, num_bub, i2,   vbub, r_bub)
         call r_pntr (allocatd, num_bub, i2, vbubin, r_bub)
         call r_pntr (allocatd, num_bub, i2,  vbubo, r_bub)
         call r_pntr (allocatd, num_bub, i2,     zm, r_bub)
         call r_pntr (allocatd, num_bub, i2,    zmo, r_bub)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rbub   = i2
!
            allocate (r_bub(siz_rbub), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rbub, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_bub
            block  = 'r_bub'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rbub, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_bub      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_bub, i2, ibubol, i_bub)
         call i_pntr (allocatd, num_bub, i2,  ibvol, i_bub)
         call i_pntr (allocatd, num_bub, i2, idalph, i_bub)
         call i_pntr (allocatd, num_bub, i2, idvbub, i_bub)
         call i_pntr (allocatd, num_bub, i2, iwilsn, i_bub)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ibub   = i2
!
            allocate (i_bub(siz_ibub), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ibub, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_bub
            block  = 'i_bub'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ibub, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_bub      = 0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for logical arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call l_pntr (allocatd, num_bub, i2, sepr8r, l_bub)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_lbub   = i2
!
            allocate (l_bub(siz_lbub), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'logical'
               write (6,100) block, siz_lbub, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            ldum   => l_bub
            block  = 'l_bub'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_lbub, 3, 2)
!
!      initialize logical arrays to false
!
         else
            l_bub      = .false.
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' bubble rise arrays.'/ &
           10x,'Length requested = ',i8/ &
           10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for bubble rise arrays. '/ &
           10x,'Variable num_bub = ',i3, ', must be > 0.')
      end subroutine allok8_bubble_rise
!
!  ===============================================================================================================================
!
!     Subroutine to allocate space and set pointers for level model arrays
!     Note: num_lvl must be defined before allok8_level_model is called
!
!     allok8_level_model is called from subroutine inbstk
!
      subroutine allok8_level_model
!      
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
!      if (.not.allocated (r_bub)) then
!         write (6,110) 
!         call fail
!         return
!      endif
!      
      if (allocated (r_lvl)) return
!      
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second 
!  pass assigns pointer arrays.
!  -------------------------------------------------------------------------------------------------------------------------------
!    
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_lvl, i2, eqliql, r_lvl)
         call r_pntr (allocatd, num_lvl, i2, eqmixl, r_lvl)
! 
!      allocate space
!
         if (.not.allocatd) then
            siz_rlvl   = i2
!
            allocate (r_lvl(siz_rlvl), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rlvl, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_lvl
            block  = 'r_level'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rlvl, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_lvl      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_lvl, i2, idxvsk, i_lvl)
         call i_pntr (allocatd, num_lvl, i2, nvolst, i_lvl)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ilvl   = i2
!
            allocate (i_lvl(siz_ilvl), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ilvl, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_lvl
            block  = 'i_lvl'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ilvl, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_lvl      = 0
         endif
      enddo
!
      return
!      
  100 format (' ******** Error allocating space for ',a8,' level model arrays (allok8_level_model).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for level model arrays (allok8_level_model).'/ &
          10x,'Subroutine allok8_level_model called before subroutine allok8_bubble_rise - programming error') 
!
      end subroutine allok8_level_model
!      
!  ===============================================================================================================================
!
!     Subroutine to allocate space and set pointers for level parameter arrays
!     Note: num_lvl must be defined before allok8_level_para is called
!
!     allok8_level_para is called from subroutine inbstk
!
      subroutine allok8_level_para
!     
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i, i1, i2, l, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!       
      if (.not.allocated (r_lvl)) then
         write (6,110) 
         call fail
         return
      endif
!      
      if (allocated (r_lpara)) return
!      
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second 
!  pass assigns pointer arrays.
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Loop over all level model to allocate space for all volume parameters and related arrays.  Note:  volumes index from 
!  all stack live in a subfile indexed by idxvsk. The begining index for volume 1 for stack 1 is idxvsk(1) = 1, volume 1 for 
!  stack 2 is ivlidx(2) = 2. 
!
      if (num_stkvol .le.0) then
         write (6,120) num_stkvol
         call fail
         return
      endif
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_stkvol, i2, botovl, r_lpara)
         call r_pntr (allocatd, num_stkvol, i2, topovl, r_lpara)
! 
!      allocate space
!
         if (.not.allocatd) then
            siz_rlpara   = i2
!
            allocate (r_lpara(siz_rlpara), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rlpara, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_lpara
            block  = 'r_lpara'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rlpara, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_lpara      = 0.0d0
         endif
      enddo
! 
!  initialize idxvsk for all stacks
!
      i1     = 1
      do i = 1, num_lvl
         idxvsk(i)  = i
      enddo
!      
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_stkvol, i2, ivlidx, i_lpara)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ilpara   = i2
!
            allocate (i_lpara(siz_ilpara), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status/=0) then
               block      = 'integer'
               write (6,100) block, siz_ilpara, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_lpara
            block  = 'i_lpara'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ilpara, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_lpara      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' stack volume parameter arrays (allok8_level_para).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for stack volume parameter arrays (allok8_level_para).'/ &
          10x,'Subroutine allok8_level_para called before subroutine allok8_level_model - programming error') 
  120 format (' ******** Error allocating space for stack volume parameter arrays (allok8_level_para).'/ &
          10x,'Sum of number of volumes for all stacks = ',i6,', must be > 0 - programming error') 
!          
      end subroutine allok8_level_para
!
      end module bubble_rise
