!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for centrifugal pump related arrays
!
!  This module also includes the following subroutines
!
!      allok8_pumps
!      allok8_motor_torque
!      allok8_hdtq_mult
!      allok8_headntorque_curves
!
!  =============================================================================================================================== 
!
      module centrifugal_pumps
!
      use kind_specs
!
      implicit none      
!                      
      integer(kind=ikd), save :: &
          num_pump, &  !  number of pumps
         siz_rpump, &  !  size of the array allocated for the real pump pointer arrays
         siz_ipump     !  size of the array allocated for the integer pump pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_pump(:)
!
!  All of the following real and integer pointer arrays are Rank (1:num_pump)
!  Space is allocated and pointers associations are made in subroutine allok8_pumps
!
      real(kind=rkd), pointer, save :: &
       cavcon(:), &  !  pump stop on elapsed time                        
        dphdm(:), &  !  partial of pump head wrt fluid mass              
       dphdmg(:), &  !  partial of pump head wrt vapor mass              
        dphdu(:), &  !  partial of pump head wrt internal energy         
       dphdul(:), &  !  partial of pump head wrt liq phase int energy    
        dphdw(:), &  !  partial of pump head wrt jun mass flow rate      
        dqpdm(:), &  !  partial of pump vol flow wrt fluid mass          
       dqpdmg(:), &  !  partial of pump vol flow wrt vap mass            
        dqpdu(:), &  !  partial of pump vol flow wrt int energy          
       dqpdul(:), &  !  partial of pump vol flow wrt liq int energy      
        dqpdw(:), &  !  partial of pump vol flow wrt jun mass flow rate  
         dtdm(:), &  !  partial of hydraulic torque wrt mass             
         dtdw(:), &  !  partial of hydraulic torque wrt flow             
        fpump(:), &  !  pump stop on maximum forward speed               
        gamma(:), &  !  implicit solution damping coefficient            
       pflowr(:), &  !  rated pump flow                                  
       pftorq(:), &  !  normalized pump frictional torque                
       pheadr(:), &  !  rated pump head                                  
       pinrta(:), &  !  pump moment of inertia                           
       pmpowr(:)     !  power to coolant due to irrecoverable losses     
!
      real(kind=rkd), pointer, save :: &
       pmtorq(:), &  !  normalized pump motor torque                     
        pomga(:), &  !  actual pump angular velocity                     
       pomgao(:), &  !  pump angular velocity from previous time step    
       pomgar(:), &  !  rated pump speed                                 
        psrat(:), &  !  ratio of initial to rated speed                  
       ptorkr(:), &  !  rated pump torque                                
        spump(:), &  !  pump stop on maximum reverse speed               
       torkf1(:), &  !  torkf1 through torkf4                            
       torkf2(:), &  !  are coefficients for                             
       torkf3(:), &  !  frictional torque as a cubic                     
       torkf4(:), &  !  function of the speed ratio.                     
       torkfr(:), &  !  frictional torque at rated speed                 
       torkmr(:), &  !  rated pump motor torque                          
       torque(:), &  !  normalized pump torque                           
        vrhoi(:)     !  rated or initial density                         
!
      integer(kind=ikd), target, allocatable, save :: i_pump(:)
      integer(kind=ikd), pointer, save :: &
          imt(:), &  !  index for pump motor torque curve                
          imz(:), &  !  memory index for motor torque curve table        
          ipc(:), &  !  pump curve set indicator                         
       ipeqst(:), &  !  flow network number to which the pump belongs    
          ipm(:), &  !  index for two-phase option (0=no - 1=yes)        
         iptm(:), &  !  index for torque multiplier curve          
          irp(:), &  !  pump reverse indicator (1 = reverse allowed)     
       itpump(:), &  !  trip number to start or shut off pump            
       ivpmpo(:), &  !  pump old volume number                           
       ivpump(:), &  !  index to pump volume 
       jdisrg(:), &  !  discharge junction index 
         jphd(:), &  !  memory index for head curve table                
        jphd4(:), &  !  memory index for single-2 phase head curve table 
         jphm(:), &  !  memory index for pump head mult. curve table     
         jptk(:), &  !  memory index for torque curve table              
        jptk4(:), &  !  memory index for single-2 phase torq curve table 
         jptm(:), &  !  memory index for pump torque mult. curve table   
       jsuctn(:), &  !  suction junction index
       olpmpn(:)     !  old pump number                                  
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Pump motor torque related table arrays
!
      integer(kind=ikd), save :: &
         sum_ptmo , &  !  total number of pump motor torque data points for all pumps
         siz_itork, &  !  size of the array allocated for the integer tork table pointer arrays
         siz_rtork     !  size of the array allocated for the real tork table pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_tork(:)
!
!  The following real pointer array is Rank (1:sum_ptmo) where sum_ptmo is the sum of the number of points in 
!       all motor torque curves.  All curves reside in array ptmo and the index for each curve is imt(i) where i 
!       is the pump index. Index into real array ptmo for each curve is saved into idxpm. idxpm is ranked to
!       2 * num_pump. 1st num_pump values inside idxpm stores indexes into ptmo for each curve, 2nd num_pump values inside
!       idxpm stores imt values for each curves.
!  Space is allocated and pointers associations are made in subroutine allok8_motor_torque
!
      real(kind=rkd), pointer, save :: &
         ptmo(:)     !  pump motor torque or pump speed vs time curves
!
      integer(kind=ikd), target, allocatable, save :: i_tork(:)
      integer(kind=ikd), pointer, save :: &                  
         idxpm(:), &  !  index into pump motor toruq array
          ntmo(:)     !  number of points in pump motor torque curve
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Pump head and torque multipier curves
!
      integer(kind=ikd), save :: &
          num_headm, &      !  total number of head multiplier points
          num_torqm, &      !  total number of torque multiplier points
          siz_rheadm, &     !  size of the array allocated for the real pump head or torque multiplier curves
          siz_iheadm        !  size of the array allocated for the integer pump head or torque multiplier curves
!
      real(kind=rkd), target, allocatable, save :: r_headm(:)
!
!   The following real pointer arrays are Rank (1:num_headm) or (1:num_torqm)
!   in either the head or torque multiplier curves (as appropriate).  The value of icurve(i) matches with
!   the value of ipm(i).
!   Space is allocated and pointers associations are made in subroutine allok8_hdtq_mult
!
      real(kind=rkd), pointer, save :: &
         phdm(:), &  !  pump head multiplier curve table                  
         ptkm(:)     !  pump torque multiplier curve table               
!
      integer(kind=ikd), target, allocatable, save :: i_headm(:)
      integer(kind=ikd), pointer, save :: &         
       icurve(:), &  !  index of pump degradation curve, ipm
       idxphm(:), &  !  index into pump head multiplier curve table
       idxptm(:), &  !  index into pump torque multiplier curve table
         nphm(:), &  !  number of data points in the pump head mult. curve
         nptm(:)     !  number of data points in the pump torque mult. curve
         
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Pump head and torque characteristic curves
!
      integer(kind=ikd), save :: &
        num_curves, &  !  number of pump curve sets
         siz_rhead, &  !  size of the array allocated for the real head pointer arrays
         siz_ihead     !  size of the array allocated for the integer head pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_head(:)
!
!  The following real pointer arrays are Rank (1:2*sum_pts) where sum_pts is the sum of the number of 
!       points in either the head or torque curves (as appropriate).  The index to a given head or torque
!       curve is idxcrv(idxp(i)+j) where i is pump index and j is the head or torque curve identifier (1-16).
!  Space is allocated and pointers associations are made in subroutine allok8_headntorque_curves
!
      real(kind=rkd), pointer, save :: &
        phead(:), &  !  array used to store head curves for each curve set
        ptork(:)     !  array used to store toque curves for each curve set
!
!  There can be 8 head and 8 torque curves that define each pump curve set - the following pointers are repeated
!  for each curve.  Curves 1 - 8 are head curves while 9 - 16 are torque curves.
!
      integer(kind=ikd), target, allocatable, save :: i_head(:)
      integer(kind=ikd), pointer, save :: &
       idxcrv(:), &  !  index to head or torque curve - phead / ptork
         idxp(:), &  !  index into curveset table - idxcrv
         npcr(:)     !  number of points in the table for this curve     
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_pumps
!
!  Subroutine to allocate space and set pointers for pump related arrays
!     Note: num_pump must be defined prior to calling subroutine allok8_pumps
!
!     allok8_pumps is called from subroutine inpump 
!
      use kind_specs 
      use restart_block_list
! 
!     Interface modules
!

      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_pump)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_pump, i2, cavcon, r_pump)
         call r_pntr (allocatd, num_pump, i2,  dphdm, r_pump)
         call r_pntr (allocatd, num_pump, i2, dphdmg, r_pump)
         call r_pntr (allocatd, num_pump, i2,  dphdu, r_pump)
         call r_pntr (allocatd, num_pump, i2, dphdul, r_pump)
         call r_pntr (allocatd, num_pump, i2,  dphdw, r_pump)
         call r_pntr (allocatd, num_pump, i2,  dqpdm, r_pump)
         call r_pntr (allocatd, num_pump, i2, dqpdmg, r_pump)
         call r_pntr (allocatd, num_pump, i2,  dqpdu, r_pump)
         call r_pntr (allocatd, num_pump, i2, dqpdul, r_pump)
         call r_pntr (allocatd, num_pump, i2,  dqpdw, r_pump)
         call r_pntr (allocatd, num_pump, i2,   dtdm, r_pump)
         call r_pntr (allocatd, num_pump, i2,   dtdw, r_pump)
         call r_pntr (allocatd, num_pump, i2,  fpump, r_pump)
         call r_pntr (allocatd, num_pump, i2,  gamma, r_pump)
         call r_pntr (allocatd, num_pump, i2, pflowr, r_pump)
         call r_pntr (allocatd, num_pump, i2, pftorq, r_pump)
         call r_pntr (allocatd, num_pump, i2, pheadr, r_pump)
         call r_pntr (allocatd, num_pump, i2, pinrta, r_pump)
         call r_pntr (allocatd, num_pump, i2, pmpowr, r_pump)
         call r_pntr (allocatd, num_pump, i2, pmtorq, r_pump)
         call r_pntr (allocatd, num_pump, i2,  pomga, r_pump)
         call r_pntr (allocatd, num_pump, i2, pomgao, r_pump)
         call r_pntr (allocatd, num_pump, i2, pomgar, r_pump)
         call r_pntr (allocatd, num_pump, i2,  psrat, r_pump)
         call r_pntr (allocatd, num_pump, i2, ptorkr, r_pump)
         call r_pntr (allocatd, num_pump, i2,  spump, r_pump)
         call r_pntr (allocatd, num_pump, i2, torkf1, r_pump)
         call r_pntr (allocatd, num_pump, i2, torkf2, r_pump)
         call r_pntr (allocatd, num_pump, i2, torkf3, r_pump)
         call r_pntr (allocatd, num_pump, i2, torkf4, r_pump)
         call r_pntr (allocatd, num_pump, i2, torkfr, r_pump)
         call r_pntr (allocatd, num_pump, i2, torkmr, r_pump)
         call r_pntr (allocatd, num_pump, i2, torque, r_pump)
         call r_pntr (allocatd, num_pump, i2,  vrhoi, r_pump)        
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rpump   = i2
!
            allocate (r_pump(siz_rpump), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rpump, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_pump
            block  = 'r_pump'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rpump, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_pump      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_pump, i2,    imt, i_pump)
         call i_pntr (allocatd, num_pump, i2,    imz, i_pump)
         call i_pntr (allocatd, num_pump, i2,    ipc, i_pump)
         call i_pntr (allocatd, num_pump, i2, ipeqst, i_pump)
         call i_pntr (allocatd, num_pump, i2,    ipm, i_pump)
         call i_pntr (allocatd, num_pump, i2,    irp, i_pump)
         call i_pntr (allocatd, num_pump, i2, itpump, i_pump)
         call i_pntr (allocatd, num_pump, i2, ivpmpo, i_pump)
         call i_pntr (allocatd, num_pump, i2, ivpump, i_pump)
         call i_pntr (allocatd, num_pump, i2, jdisrg, i_pump)
         call i_pntr (allocatd, num_pump, i2,   jphd, i_pump)
         call i_pntr (allocatd, num_pump, i2,  jphd4, i_pump)
         call i_pntr (allocatd, num_pump, i2,   jphm, i_pump)
         call i_pntr (allocatd, num_pump, i2,   jptk, i_pump)
         call i_pntr (allocatd, num_pump, i2,  jptk4, i_pump)
         call i_pntr (allocatd, num_pump, i2,   jptm, i_pump)
         call i_pntr (allocatd, num_pump, i2, jsuctn, i_pump)
         call i_pntr (allocatd, num_pump, i2, olpmpn, i_pump)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ipump   = i2
!
            allocate (i_pump(siz_ipump), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ipump, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_pump
            block  = 'i_pump'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ipump, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_pump      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' pump arrays.'/ &
           10x,'Length requested = ',i8/ &
           10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_pumps
!
!  ===============================================================================================================================
      subroutine allok8_motor_torque  
!
!  Subroutine to allocate space and set pointers for pump motor torque related arrays
!     Note: allok8_motor_torque must not be called until allok8_pumps has been called or an error will occur and processing
!           terminated.  In addition, the ntmo array must be initialized prior to calling allok8_motor_torque.  If ntmo is 
!           not initialized, no space will be allocated for the motor torque curves.
!
!     allok8_motor_torque is called from subroutine inpump 
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i, l, i2, status, point_sum
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_tork)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_pump)) then
         write (6,110) 
         call fail
         return
      end if
!      
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_pump*2, i2,  idxpm, i_tork)
         call i_pntr (allocatd, num_pump, i2,     ntmo, i_tork)                  
!
!      allocate space
!
         if (.not.allocatd) then
            siz_itork = i2
            allocate (i_tork(siz_itork), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_itork, status
               call fail
            end if
!
!        Add block to list of blocks included in restart file
!
            idum   => i_tork
            block  = 'i_tork'
            call build_restart_block_list (block, rdum, idum, ldum, siz_itork, 2, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            i_tork      = 0.0d0
         end if
!
      end do
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Note: table arrays for all motor torque (or speed vs time) data tables reside in a single array ptmo.  The beginning 
!  index for ptmo for table 1 is |imt(1)| = 1.  For the second table |imt(2)| = (|imt(1) + 2*ntmo(1)). This pattern 
!  continues for all other motor tork tables. (imt is only defined if ntmo .ne. 0)
!
!  Determine space needed for pump motor torque (or speed) curves
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, sum_ptmo, i2,   ptmo, r_tork)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rtork = i2
            allocate (r_tork(siz_rtork), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rtork, status
               call fail
            end if
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_tork
            block  = 'r_tork'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rtork, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_tork      = 0.0d0
         end if
!
!  Set indices to motor torque curves
!
         i2          = 1
!
      end do
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' pump motor torque arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for pump motor torque table arrays.'/ &
          10x,'Subroutine allok8_motor_torque called before subroutine allok8_pumps - programming error') 
!          
      end subroutine allok8_motor_torque
!
!  ===============================================================================================================================
      subroutine allok8_hdtq_mult  
!
!  Subroutine to allocate space and set pointers for pump head and torque multiplier related arrays
!     Note: allok8_hdtq_mult must not be called until allok8_pumps has been called or an error will occur and processing
!           terminated.  In addition, the nphm and mptm arrays must be initialized prior to calling allok8_hdtq_mult. 
!           If they are not initialized, no space will be allocated for the head and torque multiplier curves.
!
!     allok8_hdtq_mult is called from subroutine inpump 
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i, l, i1, i2, status, point_sum1, point_sum2
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (i_headm)) go to 2
!      
      if (.not.allocated (r_pump)) then
         write (6,110) 
         call fail
         return
      endif      
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_pump, i2, icurve, i_headm)         
         call i_pntr (allocatd, num_pump, i2, idxphm, i_headm)         
         call i_pntr (allocatd, num_pump, i2, idxptm, i_headm)         
         call i_pntr (allocatd, num_pump, i2,   nphm, i_headm)
         call i_pntr (allocatd, num_pump, i2,   nptm, i_headm)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_iheadm = i2
!
            allocate (i_headm(siz_iheadm), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_iheadm, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_headm
            block  = 'i_headm'
            call build_restart_block_list (block, rdum, idum, ldum, siz_iheadm, 2, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            i_headm      = 0
         end if
!
      end do
!      
      return
      
    2 if (allocated (r_headm)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Note: multiplier arrays for all head curves reside in array phdm.  The beginning 
!  index for phdm for curve 1 is ipm(1) = 1.  For the second curve ipm(2) = (ipm(1) + 2*nphm(1)). This pattern continues 
!  for all other head multiplier curves. (ipm is only defined if nphm .ne. 0)  The torque multiplier arrays for all 
!  torque curves reside in array ptqm.  The beginning index for ptqm for curve 1 is iptm(1) = 1.  For the second curve 
!  iptm(2) = (iptm(1) + 2*nptm(1)). This pattern continues for all other torque multiplier curves. (iptm is only defined 
!  if nptm .ne. 0)
!
!  Return if no head or torque multiplier curves are supplied
!
      if (num_headm .le. 0 .and. num_torqm .le. 0) then
         return
      end if
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_headm, i2,   phdm, r_headm)
         call r_pntr (allocatd, num_torqm, i2,   ptkm, r_headm)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rheadm = i2
!
            allocate (r_headm(siz_rheadm), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rheadm, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_headm
            block  = 'r_headm'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rheadm, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_headm      = 0.0d0
         endif
!
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' pump head and torque multipllier curve arrays.'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for pump head and torque multiplier curve arrays.'/ &
          10x,'Subroutine allok8_hdtq_mult called before subroutine allok8_pumps - programming error') 
      end subroutine allok8_hdtq_mult
!
!
!  ===============================================================================================================================
      subroutine allok8_headntorque_curves  
!
!  Subroutine to allocate space and set pointers for pump head and torque curves that comprise a pump cureve set
!     Note: allok8_headntorque_curves must not be called until allok8_pumps has been called or an error will 
!           occur and processing terminated.  allok8_headntorque_curves must be called twice, once to allocate
!           the integer pointer arrays and a second time to allocate the head and torque curve arrays.  After 
!           the first call, the size or each head and torque curve for each curve set must be defined in the 
!           curve set array npcr.
!
!     allok8_headntorque_curves is called from subroutine inpump 
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i, j, l, i2, status, point_sum1, point_sum2, sum_ncurve
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_head)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (.not.allocated (r_pump)) then
         write (6,110) 
         call fail
         return
      endif
!
      if (allocated (i_head)) go to 10
!
      sum_ncurve = 16*num_curves
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, sum_ncurve, i2, idxcrv, i_head)
         call i_pntr (allocatd, num_curves, i2,   idxp, i_head)
         call i_pntr (allocatd, sum_ncurve, i2,   npcr, i_head)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ihead   = i2
!
            allocate (i_head(siz_ihead), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ihead, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_head
            block  = 'i_head'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ihead, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_head      = 0
         endif
      enddo
!
      return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Loop over all pump curve sets to determine the space required for head and torque curves.  Note: all head curves 
!  reside in array phead and all torque curves reside in array ptork.  The beginning index for each head curve within 
!  a given curve set is idxcrv(ipc(i)+n-1) and the number of points is npcr(ipc(i)+n-1), where i is the pump number 
!  and n is the head curve number (1 - 8).  The index for the corresponding torque curve is idxcrv(ipc(i)+n+7) and 
!  the number of points is npcr(ipc(i)+n+7).
!
   10 continue
      if (.not.allocated (r_pump)) then
         write (6,110) 
         call fail
         return
      endif
!
!  Determine space needed for head and torque multiplier curves
!
      point_sum1 = 0
      point_sum2 = 0
      i2         = 1
!
      do i = 1, num_curves
         do j = 1, 8
            point_sum1 = point_sum1 + npcr(i2+j-1)
            point_sum2 = point_sum2 + npcr(i2+j+7)
         end do
         i2 = i2 + 16
      end do
! 
!  Error if no head or torque curves are used
!
      if (point_sum1 .le.0 .or. point_sum2.le.0) then
         write (6,120)
         call fail
         return
      endif
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, 2*point_sum1, i2,   phead, r_head)
         call r_pntr (allocatd, 2*point_sum2, i2,   ptork, r_head)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rhead = i2
!
            allocate (r_head(siz_rhead), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rhead, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_head
            block  = 'r_head'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rhead, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_head      = 0.0d0
         end if            
      enddo
!
      return
!
  100 format (' ******** ERROR ALLOCATING SPACE FOR ',a8,' PUMP HEAD AND TORQUE CURVE ARRAYS(ALLOK8_HEADNTORQUE_CURVES).'/ &
          10x,'LENGTH REQUESTED = ',i8/ &
          10x,'STATUS FLAG RETURNED FROM ALLOCATE = ',i3)
  110 format (' ******** ERROR ALLOCATING SPACE FOR PUMP HEAD AND TORQUE CURVES.'/ &
          10x,'SUBROUTINE allok8_headntorque_curves CALLED BEFORE SUBROUTINE allok8_pumps - PROGRAMMING ERROR') 
  120 format (' ******** ERROR, ZERO LENGTH SPACE COMPUTED FOR PUMP HEAD AND TORQUE CURVES - PROGRAMMING ERROR - ', &
              'IN SUBROUTINE (allok8_headntorque_curves)') 
      end subroutine allok8_headntorque_curves
!
      end module centrifugal_pumps
