!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for heat conductor related arrays
!
!  This module also includes the following subroutines
!
!      allok8_conductors
!      allok8_tparrays
!      allok8_conductor_geometries
!      allok8_geometry_regions
!      allok8_stacks
!      allok8_alfa
!
!  ===============================================================================================================================
!
      module conductors
!
      use kind_specs
!
      integer(kind=ikd), save :: &
           num_cond, &  !  number of heat conductors
           num_auxc, &  !  number of auxiliary conductors (used by two region enthalpy transport)
          siz_rcond, &  !  size of the array allocated for the real 1D conductor pointer arrays
          siz_icond     !  size of the array allocated for the integer 1D conductor pointer arrays
!
!  All of the following real and integer pointer arrays are Rank (1:num_cond)
!  Space is allocated and pointers associations are made in subroutine allok8_conductors
!
      real(kind=rkd), target, allocatable, save :: r_cond(:)  !  real target array for heat conductor pointer arrays
      real(kind=rkd), pointer, save :: &
        celev(:), &  !  elevation of the midpoint of the conductor referenced from bottom of vol        
         cndl(:), &  !  conductor length                                 
           se(:), &  !  stored energy in conductor                       
         slen(:), &  !  equivalent length of heat cond                   
         vols(:), &  !  volume of heat slab                              
           z1(:), &  !  starting length for conductor where cond         
           z2(:)     !  ending length of conductor used for cond         

      integer(kind=ikd), target, allocatable, save :: i_cond(:)
      integer(kind=ikd), pointer, save :: &
       icflg1(:), &  !  flag to indicate vertical orientation of tubes (1 implies vert)
       icflg2(:), &  !  not used, reserved for future use                
        icfwh(:), &  !  flag to indicate use of heat conductor as feedwater heater                                 
        icond(:), &  !  conductor flag for condensation                  
!        idxap(:), &  !  index of file containing ap for each node        
       idxaux(:), &  !  index for auxilary conductor (w shs model)
        idxtp(:), &  !  index of file containing tp for each node        
       idxtpc(:), &  !  index of file containing itpc for each region    
         igom(:), &  !  geometry index                                   
        ihxqf(:), &  !  flag to indicate use of heat conductor as a heat exchanger                                        
       ireqst(:), &  !  flow network number for secondary side of the steam gen ( >0 only if ihxqf>0 )           
         isco(:), &  !  core number                                    
        lcond(:), &  !  flag indicating use of local cond model for this conductor. 0 = no, 1 = right, 2 = left
        nnode(:), &  !  number of temperature nodes in the heat slab      
       ntubes(:), &  !  number of tubes for mode 15 heat transfer corr.  
       olslbn(:)     !  old conductor number                             
!
!     Following pointer is used only during two region enthalpy transport and 
!     allocated inside subroutine "in2ret"
      integer(kind=ikd), allocatable, save :: idslb(:)
!  All of the following real and integer pointer arrays are Rank (1:num_cond,1:2).  Space is allocated 
!  and pointers associations are made in subroutine allok8_conductors. For all of the following 2D arrays 
!  the second subscript is used to define the "left" side (=1) and the "right" side (=2) for a conductor 
!  variable, unless otherwise noted.
!
      integer(kind=ikd), save :: &
         siz_r2cond, &  !  size of the array allocated for the real 2D conductor pointer arrays
         siz_i2cond     !  size of the array allocated for the integer 2D conductor pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r2_cond(:,:)  !  real target array for heat conductor 2D pointer arrays
      real(kind=rkd), pointer, save :: &
         ahtl(:,:), &  !  heat tran area at left conductor surface         
        alphl(:,:), &  !  heat flux constant a - qw = a*ts + b       
        avedl(:,:), &  !  average fluid density at boundary
           az(:,:), &  !  coefficient for chf correlations
          az1(:,:), &  !  coefficient for chf correlations                 
           bz(:,:), &  !  coefficient for chf correlations                        
         chnl(:,:), &  !  channel length on left of cond                   
       cpleft(:,:), &  !  fluid heat capacity at boundary
           cz(:,:), &  !  coefficient for chf correlations
         dhel(:,:), &  !  heated equivalent diameter at boundary 
       dqldhw(:,:), &  !  partial of wall heat flux wrt bnd vol enth    
       dqldpb(:,:), &  !  partial of wall heat flux wrt bnd vol pres           
       dqldtw(:,:), &  !  partial of wall heat flux wrt wall temp     
           ez(:,:), &  !  coefficient for chf correlations                 
          ez1(:,:), &  !  coefficient for chf correlations       
         fchl(:,:), &  !  critical heat flux at boundary
           fz(:,:), &  !  coefficient for chf correlations                          
           gl(:,:), &  !  coolant flow rate at boundary
         hdml(:,:), &  !  hydraulic diameter of volume at boundary
         htcl(:,:), &  !  heat transfer coefficient at boundary
           hz(:,:), &  !  coefficient for chf correlations                                 
         phil(:,:), &  !  heat flux at boundary
         phol(:,:), &  !  heat flux at boundary (old time step)   
       tixncl(:,:), &  !  surface interface temp for condensation at boundary
           tl(:,:), &  !  sink temperature, f  at boundary                         
        tlocl(:,:), &  !  local temperature at boundary
         wqcl(:,:), &  !  heat trans rate to fluid at boundary
        xlocl(:,:), &  !  local quality  at boundary
          wed(:,:)     !  wetted equivalent diameter, in                           

      integer(kind=ikd), target, allocatable, save :: i2_cond(:,:)  !  integer target array for heat conductor 2D pointer arrays
      integer(kind=ikd), pointer, save :: &
         ibcl(:,:), &  !  boundary condition indicator                
         ihtl(:,:), &  !  heat trans mode  at boundary
         imcl(:,:), &  !  heat transfer correlation indicator at boundary
         injl(:,:), &  !  index to inlet junction for volume at boundary
         ivsl(:,:), &  !  index of volume at boundary
       ivslol(:,:)     !  old volume number at boundary
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for temperature and ap related heat conduction arrays
!
      integer(kind=ikd), save :: &
           node_sum, &  !  sum of total number of nodes for all conductors
            reg_sum, &  !  sum of total number of regions for all conductors 
         siz_itemps, &  !  size of integer array allocated for temperature and related heat conduction pointer arrays
         siz_rtemps, &  !  size of real array allocated for temperature and related heat conduction pointer arrays
          max_nodes     !  maximum number of nodes in any heat conductor
!
      real(kind=rkd), target, allocatable, save :: &
            r_temps(:)  !  real target array for temperature and related pointer arrays
!
!  The tp, oldtp, ap and oldap real pointer arrays are Rank (1:node_sum) where node_sum is the sum of the number of 
!       nodes for all heat conductors.  Index idxtp(i) points to the beginning location of the tp, ap, oldtp and 
!       oldap arrays for each conductor, where i in the index for the conductor.
!  Space is allocated and pointers associations are made in subroutine allok8_tparrays
!
      real(kind=rkd), pointer, save :: &
           ap(:), &  !  -2*surface weight*thermal conductivity (for node xx)       
        oldap(:), &  !  previous time step -2*surface weight*thermal conductivity (for node xx)            
        oldtp(:), &  !  previous time step heat conductor node xx temperature                              
           tp(:)     !  heat conductor node xx temperature                                
!
!  The following real pointer arrays are Rank (1:max_nodes) where max_nodes is the maximum number of nodes for 
!       all conductors.  They are the work space used to store the volumetric heat generation and material
!       properties as each conductor solution is performed (same space used for all conductors).
!  Space is allocated and pointers associations are made in subroutine allok8_tparrays
!
      real(kind=rkd), pointer, save :: &
           cr(:), &  !  volumetric heat capacity at node xx                  
           qn(:), &  !  internal heat source for space mesh xx               
           tk(:)     !  thermal conductivity at node xx                      
!
      integer(kind=ikd), target, allocatable, save :: &
            i_temps(:)  !  integer target array for temperature and related pointer arrays
!
!  The itpc, itpk, itpx pointer arrays are Rank (1:reg_sum) where reg_sum is the sum of the number of 
!  regions for all heat conductors geometries.  Index idxtpc(i) points to the beginning location of the itpc, itpk, itpx
!  arrays for each region, where i in the index for the conductor.
!  Space is allocated and pointers associations are made in subroutine allok8_tparrays
!
      integer(kind=ikd), pointer, save :: &
         itpc(:), &  !  current position in tpc table for region xx        
         itpk(:), &  !  current position in tpk table for region xx        
         itpx(:)     !  current position in tpx table for region xx        
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for heat conductor geometry related arrays
!
      integer(kind=ikd), save :: &
          num_geom, &  !  number of geometry decsriptions
         siz_rgeom, &  !  size of the array allocated for the real geom pointer arrays
         siz_igeom     !  size of the array allocated for the integer geom pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_geom(:)
!
!  All of the following real and integer pointer arrays are Rank (1:num_geom)
!  Space is allocated and pointers associations are made in subroutine allok8_conductor_geometry
!
      real(kind=rkd), pointer, save :: &
                      asul(:), &  !  left cond surface area per unit height            
                      asur(:), &  !  right cond surface area per unit height           
                        xo(:)     !  inside radius                                     
!
      integer(kind=ikd), target, allocatable, save :: i_geom(:)
      integer(kind=ikd), pointer, save :: &
          idxgp(:), &  !  index into array containing region conductivity multiplier          
           idxr(:), &  !  index into array containing geometry region variables     
           idxs(:), &  !  index into surface area weight array  
          idxvp(:), &  !  index into nodal volume array
             ig(:), &  !  geometry type, 1-rectangular, 2-cylindrical       
             nr(:), &  !  number of regions                                 
            nsi(:)     !  node at right surface                             
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Region related arrays
!
      integer(kind=ikd), save :: &
          sum_reg, &  !  sum of regions for all geometries
          sum_nod, &  !  sum of number of nodes in all regions of all geometries
         siz_rregion, &  !  size of the array allocated for the real region pointer arrays
         siz_iregion     !  size of the array allocated for the integer region pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_region(:)
!
!  All of the following real and integer pointer arrays are Rank (1:sum_reg) where sum_reg is the sum 
!     of the number of regions for all geometry types.  The beginning index for each geometry type 
!     for all arrays below is idxr(i) where i is the index for the geometry type (except for s array, its index is stored in idxs)
!  Space is allocated and pointers associations are made in subroutine allok8_geometry_regions
!
      real(kind=rkd), pointer, save :: &
         gmul(:), &  !  region thermal conductivity multiplier , gmul > 0 = multiplier, gmul < 0 = index of control block
         pofr(:), &  !  fraction of power in region of core heat cond      
            s(:), &  !  surface area weight of region xx                      
           vp(:), &  !  volume of region xx                                  
         vreg(:), &  !  volume of region per foot of length                
          vri(:), &  !  volume ratio at interface                          
         xdcr(:)     !  cold radius or distance to right surface of region 
!
      integer(kind=ikd), target, allocatable, save :: i_region(:)
      integer(kind=ikd), pointer, save :: &
         igap(:), &  !  gap indicator                                            
          ikc(:), &  !  material index                                     
           ni(:)     !  node at interface                                  
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Conductor stack related arrays
!
      integer(kind=ikd) num_stk, &   !  number of heat conductor stacks
                       siz_istk      !  size of the array allocated for the integer stk pointer arrays
!
      integer(kind=ikd), target, allocatable, save :: i_stk(:)
!
!  The following integer pointer arrays are Rank (1:num_stk)
!  Space is allocated and pointers associations are made in subroutine allok8_stacks
!
      integer(kind=ikd), pointer, save :: &
         ishd(:), &  !  local condition option flag                      
         issb(:), &  !  conductor number of first cond in stack          
         lens(:)     !  number of conductors in stack                    
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  
!  Arrays to communicating between  ENTRAN2 and TEMPSG subroutine
!
      integer(kind=ikd), save :: &          
         siz_ralfa     !  size of the array allocated for the real tempsg pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_alfa(:)
!
!  All of the following real and integer pointer arrays are Rank (1:num_cond) where num_cond is the number
!     of conductors.  
!  Space is allocated and pointers associations are made in subroutine allok8_alfa
!
      real(kind=rkd), pointer, save :: &
          alfa1(:),  &  !
          alfa2(:),  &  !
          alfa3(:),  &  !
          beta1(:),  &  !      
          beta2(:),  &  !
          beta3(:)      !
!          
      contains
!
!  ===============================================================================================================================
      subroutine allok8_conductors
!
!  Subroutine to allocate space and set pointers for heat conductor related arrays
!     Note: num_cond must be defined before allok8_conductors is called
!
!     allok8_conductors is called from subroutine inslab
! 
      use kind_specs
      use restart_block_list
!
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      use s_r2_pntr
      use s_i2_pntr
!      
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, n2d, status
      logical(kind=lkd) allocatd
!
      parameter (n2d    = 2)      
!
      real(kind=rkd),    pointer :: rdum(:), r2dum(:,:)   !  dummy real pointers
      integer(kind=ikd), pointer :: idum(:), i2dum(:,:)   !  dummy integer pointers
      logical(kind=lkd), pointer :: ldum(:), l2dum(:,:)   !  dummy logical pointer
!
      if (num_cond .le. 0) then
         write (6,110) num_cond
         call fail
         return
      endif
!
      if (allocated (r_cond)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second 
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
!     setup 1D pointer arrays
!
         call r_pntr (allocatd, num_cond, i2,  celev, r_cond)
         call r_pntr (allocatd, num_cond, i2,   cndl, r_cond)
         call r_pntr (allocatd, num_cond, i2,     se, r_cond)
         call r_pntr (allocatd, num_cond, i2,   slen, r_cond)
         call r_pntr (allocatd, num_cond, i2,   vols, r_cond)
         call r_pntr (allocatd, num_cond, i2,     z1, r_cond)
         call r_pntr (allocatd, num_cond, i2,     z2, r_cond)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rcond   = i2
!
            allocate (r_cond(siz_rcond), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rcond, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_cond
            block  = 'r_cond'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rcond, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_cond      = 0.0d0
         endif
         i2       = 0
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!
         call r2_pntr (allocatd, num_cond, n2d, i2,   ahtl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,  alphl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,  avedl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,     az, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,    az1, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,     bz, r2_cond)                  
         call r2_pntr (allocatd, num_cond, n2d, i2,   chnl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2, cpleft, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,     cz, r2_cond)         
         call r2_pntr (allocatd, num_cond, n2d, i2,   dhel, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2, dqldhw, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2, dqldpb, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2, dqldtw, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,     ez, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,    ez1, r2_cond)         
         call r2_pntr (allocatd, num_cond, n2d, i2,   fchl, r2_cond)         
         call r2_pntr (allocatd, num_cond, n2d, i2,     fz, r2_cond)                  
         call r2_pntr (allocatd, num_cond, n2d, i2,     gl, r2_cond)         
         call r2_pntr (allocatd, num_cond, n2d, i2,   hdml, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,   htcl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,     hz, r2_cond)         
         call r2_pntr (allocatd, num_cond, n2d, i2,   phil, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,   phol, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2, tixncl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,     tl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,  tlocl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,   wqcl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,  xlocl, r2_cond)
         call r2_pntr (allocatd, num_cond, n2d, i2,    wed, r2_cond)         
!
!      allocate space
!
         if (.not.allocatd) then
            siz_r2cond   = i2
!
            allocate (r2_cond(siz_r2cond,n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rcond, status
               call fail
               return
            end if
!
!        Add block to list of blocks included in restart file
!
            r2dum  => r2_cond
            block  = 'r2_cond'
!            
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_r2cond, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r2_cond      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second 
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
!     setup 1D pointer arrays
!
         call i_pntr (allocatd, num_cond, i2, icflg1, i_cond)
         call i_pntr (allocatd, num_cond, i2, icflg2, i_cond)
         call i_pntr (allocatd, num_cond, i2,  icfwh, i_cond)
         call i_pntr (allocatd, num_cond, i2,  icond, i_cond)
!         call i_pntr (allocatd, num_cond, i2,  idxap, i_cond)
         call i_pntr (allocatd, num_cond, i2, idxaux, i_cond)
         call i_pntr (allocatd, num_cond, i2,  idxtp, i_cond)
         call i_pntr (allocatd, num_cond, i2, idxtpc, i_cond)
         call i_pntr (allocatd, num_cond, i2,   igom, i_cond)
         call i_pntr (allocatd, num_cond, i2,  ihxqf, i_cond)
         call i_pntr (allocatd, num_cond, i2, ireqst, i_cond)
         call i_pntr (allocatd, num_cond, i2,   isco, i_cond)
         call i_pntr (allocatd, num_cond, i2,  lcond, i_cond)
         call i_pntr (allocatd, num_cond, i2,  nnode, i_cond)
         call i_pntr (allocatd, num_cond, i2, ntubes, i_cond)
         call i_pntr (allocatd, num_cond, i2, olslbn, i_cond)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_icond   = i2
!
            allocate (i_cond(siz_icond), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_icond, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_cond
            block  = 'i_cond'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_icond, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_cond      = 0
         endif
!         
         i2       = 0
!
!     setup 2D pointer arrays (second dimension is fixed at 2)
!
         call i2_pntr (allocatd, num_cond, n2d, i2,   ibcl, i2_cond)
         call i2_pntr (allocatd, num_cond, n2d, i2,   ihtl, i2_cond)
         call i2_pntr (allocatd, num_cond, n2d, i2,   imcl, i2_cond)
         call i2_pntr (allocatd, num_cond, n2d, i2,   injl, i2_cond)
         call i2_pntr (allocatd, num_cond, n2d, i2,   ivsl, i2_cond)
         call i2_pntr (allocatd, num_cond, n2d, i2, ivslol, i2_cond)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_i2cond   = i2
!
            allocate (i2_cond(siz_i2cond,n2d), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_i2cond, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            i2dum  => i2_cond
            block  = 'i2_cond'
!            
            call build_restart_block_list_2d (block, r2dum, i2dum, l2dum, siz_i2cond, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i2_cond      = 0
         endif
      enddo
!
  100 format (' ******** Error allocating space for ',a8,' heat conductor arrays (allok8_conductors).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for heat conductor arrays (allok8_conductors). '/ &
           10x,'Variable num_cond = ',i3, ', must be > 0.')
      end subroutine allok8_conductors
!
!  ===============================================================================================================================
      subroutine allok8_tparrays  
!
!  Subroutine to allocate space and set pointers for heat conduction related arrays - temperatures, etc.
!     Note: alok8_temperatures must not be called until allok8_conductors has been called and the nnode array 
!           initialized.
!
!     allok8_temperatures is called from subroutine inslab
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) i, l, i1, i2, status 
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (i_temps)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second 
!  pass assigns pointer arrays.
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Loop over all heat conductors to allocate space for all temperature and related arrays.  Note: temperature for all heat 
!  conductors live in a single array tp.  The beginning index for the temperature for conductor 1 is idxtp(1) = 1.  For the 
!  second conductor idxtp(2) = (idxtp(1) + nnode(1)). This pattern continues for all other conductors.  All other related 
!  arrays are use idxtp to define the beginning index.
!
      if (.not.allocated (r_cond)) then
         write (6,110) 
         call fail
         return
      endif
!
      if (node_sum .le.0) then
         write (6,120) node_sum
         call fail
         return
      endif
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd,  node_sum, i2,     ap, r_temps)
         call r_pntr (allocatd,  node_sum, i2,  oldap, r_temps)
         call r_pntr (allocatd,  node_sum, i2,  oldtp, r_temps)
         call r_pntr (allocatd,  node_sum, i2,     tp, r_temps)                  
         call r_pntr (allocatd, max_nodes, i2,     cr, r_temps)                  
         call r_pntr (allocatd, max_nodes, i2,     qn, r_temps)         
         call r_pntr (allocatd, max_nodes, i2,     tk, r_temps)         
! 
!      allocate space
!
         if (.not.allocatd) then
            siz_rtemps   = i2
!
            allocate (r_temps(siz_rtemps), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rtemps, status
               call fail
               return
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_temps
            block  = 'r_temps'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rtemps, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_temps      = 0.0d0
         endif
      enddo
!
!  initialize idxtp for all conductors
!
      i1     = 1
!      
      do i = 1, num_cond
         idxtp(i)  = i1
         i1        = i1 + nnode(i)
      end do
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second 
!  pass assigns pointer arrays.
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Loop over all heat conductors to allocate space for all itpc, itpk, itpx related arrays.  
!  The beginning index for the itpc, itpk, itpx for conductor 1 is idxtpc(1) = 1.  For the 
!  second conductor idxtpc(2) = (idxtp(1) + nnode(1)). This pattern continues for all other conductors.  All other related 
!  arrays are use idxtp to define the beginning index.
!
      if (.not.allocated (r_cond)) then
         write (6,110) 
         call fail
         return
      endif
!
      if (reg_sum .le.0) then
         write (6,120) reg_sum
         call fail
         return
      end if
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, reg_sum, i2,   itpc, i_temps)
         call i_pntr (allocatd, reg_sum, i2,   itpk, i_temps)
         call i_pntr (allocatd, reg_sum, i2,   itpx, i_temps)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_itemps   = i2
!
            allocate (i_temps(siz_itemps), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_itemps, status
               call fail
               return
            end if
!
!        Add block to list of blocks included in restart file
!
            idum   => i_temps
            block  = 'i_temps'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_itemps, 2, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            i_temps      = 0
         endif
      enddo
! 
      return
!
  100 format (' ******** Error allocating space for ',a8,' heat conductor temperature arrays (allok8_temperatures).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for heat conductor temperature arrays (allok8_temperatures).'/ &
          10x,'Subroutine allok8_temperatures called before subroutine allok8_conductors - programming error') 
  120 format (' ******** Error allocating space for heat conductor temperature arrays (allok8_temperatures).'/ &
          10x,'Sum of number of temperature nodes for all conductors = ',i6,', must be > 0 - programming error') 
!          
      end subroutine allok8_tparrays
!
!  ===============================================================================================================================
      subroutine allok8_conductor_geometries
!
!  Subroutine to allocate space and set pointers for conductor geomtry related arrays
!     allok8_conductor_geometries is called from subroutine ingeom
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_geom)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_geom, i2,   asul, r_geom)
         call r_pntr (allocatd, num_geom, i2,   asur, r_geom)
         call r_pntr (allocatd, num_geom, i2,     xo, r_geom)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rgeom   = i2
!
            allocate (r_geom(siz_rgeom), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rgeom, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_geom
            block  = 'r_geom'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rgeom, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_geom      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_geom, i2,  idxgp, i_geom)
         call i_pntr (allocatd, num_geom, i2,   idxr, i_geom)
         call i_pntr (allocatd, num_geom, i2,   idxs, i_geom)         
         call i_pntr (allocatd, num_geom, i2,  idxvp, i_geom)         
         call i_pntr (allocatd, num_geom, i2,     ig, i_geom)
         call i_pntr (allocatd, num_geom, i2,     nr, i_geom)
         call i_pntr (allocatd, num_geom, i2,    nsi, i_geom)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_igeom   = i2
            allocate (i_geom(siz_igeom), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_igeom, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_geom
            block  = 'i_geom'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_igeom, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_geom      = 0
         endif
      enddo
!
!  Initialize pointer indices to region arrays for each geometry
!
!      i2     = 1
!
!      do l = 1, num_geom
!         idxr(l)   = i2
!         i2        = i2 + nr(l)
!      enddo
!      
      return
!
  100 format (' ******** Error allocating space for ',a8,' heat conductor geometry arrays. (allok8_conductor_geometries)'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_conductor_geometries
!
!  ===============================================================================================================================
      subroutine allok8_geometry_regions
!
!  Subroutine to allocate space and set pointers for conductor geomtry related arrays
!     Note: alok8_geometry_regions must not be called until allok8_conductor_geometries has been called and the 
!           nr array initialized.
!
!     allok8_geometry_regions is called from subroutine ingeom
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_region)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Loop over all conductor geometries to determine space required for all region related arrays.  Note: all real arrays, 
!  e.g. pofr live in a single array pofr.  The beginning index for the power fraction for geometry 1 is idxr(1) = 1.  
!  For the second geometry idxr(2) = (idxr(1) + nr(1)). This patter continues for all other geometriess.  All other 
!  region related arrays also use idxr to define the beginning index (separated indices are not required for each array).
!
      if (.not.allocated (r_geom)) then
         write (6,110) 
         call fail
         return
      endif
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, sum_reg, i2,   gmul, r_region)
         call r_pntr (allocatd, sum_reg, i2,   pofr, r_region)
         call r_pntr (allocatd, sum_nod, i2,      s, r_region)
         call r_pntr (allocatd, sum_nod, i2,     vp, r_region)
         call r_pntr (allocatd, sum_reg, i2,   vreg, r_region)
         call r_pntr (allocatd, sum_reg, i2,    vri, r_region)
         call r_pntr (allocatd, sum_reg, i2,   xdcr, r_region)
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_rregion = i2
!
            allocate (r_region(siz_rregion), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rregion, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_region
            block  = 'r_region'            
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_rregion, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_region      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, sum_reg, i2,   igap, i_region)
         call i_pntr (allocatd, sum_reg, i2,    ikc, i_region)
         call i_pntr (allocatd, sum_reg, i2,     ni, i_region)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_iregion = i2
!
            allocate (i_region(siz_iregion), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_iregion, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_region
            block  = 'i_region'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_iregion, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_region      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' heat conductor geometry region arrays (allok8_geometry_regions).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for conductor geometry region arrays(allok8_geometry_regions).'/ &
          10x,'Subroutine allok8_geometry_regions called before subroutine allok8_conductor_geometry - programming error') 
  120 format (' ******** Error allocating space for heat conductor reometry region arrays (allok8_geometry_regions).'/ &
          10x,'Sum of number of regions for all conductor geometries = ',i6,', must be > 0 - programming error') 
      end subroutine allok8_geometry_regions
!
!  ===============================================================================================================================
      subroutine allok8_stacks
!
!  Subroutine to allocate space and set pointers for heat conductor stack related arrays
!     allok8_stacks is called from subroutine inslab
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_stk, i2,   ishd, i_stk)
         call i_pntr (allocatd, num_stk, i2,   issb, i_stk)
         call i_pntr (allocatd, num_stk, i2,   lens, i_stk)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_istk   = i2
!
            allocate (i_stk(siz_istk), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_istk, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_stk
            block  = 'i_stk'
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_istk, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_stk      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' heat conductor stack arrays (allok8_stacks).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_stacks
!
!  ===============================================================================================================================
!
      subroutine allok8_alfa
!
!  Subroutine to allocate space and set pointers for tempsg related arrays (alfas and betas)
!
!     allok8_alfa is called from subroutine in2ret
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_alfa)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_cond, i2,  alfa1, r_alfa)
         call r_pntr (allocatd, num_cond, i2,  alfa2, r_alfa)
         call r_pntr (allocatd, num_cond, i2,  alfa3, r_alfa)
         call r_pntr (allocatd, num_cond, i2,  beta1, r_alfa)
         call r_pntr (allocatd, num_cond, i2,  beta2, r_alfa)
         call r_pntr (allocatd, num_cond, i2,  beta3, r_alfa)                                                                                 
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_ralfa = i2
!
            allocate (r_alfa(siz_ralfa), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_ralfa, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_alfa
            block  = 'r_alfa'            
!            
            call build_restart_block_list (block, rdum, idum, ldum, siz_ralfa, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_alfa      = 0.0d0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' tempsg related arrays (allok8_alfa).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
!
      end subroutine allok8_alfa
!
      end module conductors
