!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for core related arrays
!
!  This module also includes the following subroutines
!
!     allok8_core
!
!  ===============================================================================================================================
!
      module core
!
      use kind_specs
!
      integer(kind=ikd), save :: &
            num_core, &  !  number of core sections
           siz_rcore, &  !  size of the array allocated for the real core pointer arrays
           siz_icore     !  size of the array allocated for the integer core pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_core(:)
!
!  All of the following real and integer pointer arrays are Rank (1:num_core)
!  Space is allocated and pointers associations are made in subroutine allok8_core
!
      real(kind=rkd), pointer, save :: &
         clti(:), &  !  initial clad thickness                           
          ctr(:), &  !  depth of mw reac penetration at end of time step 
        ctrin(:), &  !  internal depth of m-w reaction                   
         ctrl(:), &  !  depth of mw reac penetration at strt of time step
       dhfuel(:), &  !  difference between current fuel enthalpy and initial fuel enthalpy
           fq(:), &  !  power generation in core heat conductor          
        hfuel(:), &  !  fuel enthalpy at reference temperature (cal/gm)
       hfuel0(:), &  !  initial fuel enthalpy (at 0.0 time step)
           mq(:), &  !  moderator heating rate  (btu/hr)                 
        mqbyp(:), &  !  bypass moderator heating rate  (btu/hr)          
        qdmod(:), &  !  core section moderator delayed power fraction    
        qfrac(:), &  !  fraction of power in core section                
       qfracr(:), &  !  frac of powr gen in regs irgf through irgl - used as intrnl heat src (1-d kin only)
         qmwr(:), &  !  heat generated by m-w reaction (btu/hr)          
        qpmod(:), &  !  core section moderator prompt power fraction     
          rro(:), &  !  original fuel pin radius                         
         spv0(:), &  !  time=0 specific volume for adjacent control volume
           tm(:), &  !  average temperature of core heat conductor       
           ts(:)     !  surface temperature of core heat conductor       
!
      integer(kind=ikd), target, allocatable, save :: i_core(:)
      integer(kind=ikd), pointer, save :: &
         imht(:), &  !  memory index for moderator heating weighting factor table                                     
         irgf(:), &  !  beginning neutronic reg num from which this core cond obtains power (1-d kin only)
         irgl(:), &  !  last neutronic region number from which this core cond obtains power (1-d kin only)
         islb(:), &  !  old conductor number                             
       islbnw(:), &  !  new conductor number                             
        ivbyp(:)     !  volume index for bypass heating deposition       
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_core
!
!  Subroutine to allocate space and set pointers for core related arrays
!
!     allok8_core is called from subroutine incore
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_core)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_core, i2,   clti, r_core)
         call r_pntr (allocatd, num_core, i2,    ctr, r_core)
         call r_pntr (allocatd, num_core, i2,  ctrin, r_core)
         call r_pntr (allocatd, num_core, i2,   ctrl, r_core)
         call r_pntr (allocatd, num_core, i2, dhfuel, r_core)
         call r_pntr (allocatd, num_core, i2,     fq, r_core)
         call r_pntr (allocatd, num_core, i2,  hfuel, r_core)
         call r_pntr (allocatd, num_core, i2, hfuel0, r_core)
         call r_pntr (allocatd, num_core, i2,     mq, r_core)
         call r_pntr (allocatd, num_core, i2,  mqbyp, r_core)
         call r_pntr (allocatd, num_core, i2,  qdmod, r_core)
         call r_pntr (allocatd, num_core, i2,  qfrac, r_core)
         call r_pntr (allocatd, num_core, i2, qfracr, r_core)
         call r_pntr (allocatd, num_core, i2,   qmwr, r_core)
         call r_pntr (allocatd, num_core, i2,  qpmod, r_core)
         call r_pntr (allocatd, num_core, i2,    rro, r_core)
         call r_pntr (allocatd, num_core, i2,   spv0, r_core)
         call r_pntr (allocatd, num_core, i2,     tm, r_core)
         call r_pntr (allocatd, num_core, i2,     ts, r_core)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rcore   = i2
!
            allocate (r_core(siz_rcore), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rcore, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_core
            block  = 'r_core'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rcore, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_core      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_core, i2,   imht, i_core)
         call i_pntr (allocatd, num_core, i2,   irgf, i_core)
         call i_pntr (allocatd, num_core, i2,   irgl, i_core)
         call i_pntr (allocatd, num_core, i2,   islb, i_core)
         call i_pntr (allocatd, num_core, i2, islbnw, i_core)
         call i_pntr (allocatd, num_core, i2,  ivbyp, i_core)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_icore   = i2
!
            allocate (i_core(siz_icore), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_icore, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_core
            block  = 'i_core'
            call build_restart_block_list (block, rdum, idum, ldum, siz_icore, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_core      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' core section arrays. (allok8_core)'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
      end subroutine allok8_core
!
      end module core
