!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for the dynamic gap model related arrays
!
!  This module also includes the following subroutines
!
!      allok8_gap
!
!  ===============================================================================================================================
!
      module dynamic_gap
!
      use kind_specs
!
      integer(kind=ikd), save :: & 
          num_gap, &  !  number of gap models (always = 1)
          stk_cnd, &  !  sum of total number of conductors in fuel rod stack 
         siz_rgap, &  !  size of the array allocated for the real gap pointer arrays
         siz_igap     !  size of the array allocated for the integer gap pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_gap(:)
!
!  All of the following real and integer pointer arrays are Rank (1:num_gap)
!  Space is allocated and pointers associations are made in subroutine allok8_gap_conductance
!
      real(kind=rkd), pointer, save :: &
        caexp(:), &  !  clad axial expansion (ft)                        
        fathx(:), &  !  fuel axial thermal expansion (ft)                
       ftdens(:), &  !  fuel theoretical density fraction                
       gasmol(:), &  !  moles of fill gas in nuclear fuel rod            
       gsfrac(:), &  !  molar fraction of fill gas components            
         pgas(:), &  !  fill gas pressure for nuclear fuel rod (psia)    
        roufc(:), &  !  roughness of clas inner surface (in)             
        rouff(:), &  !  surface roughness of the fuel pellet (in)        
        vplen(:)     !  volume of gas plenum for fuel rod (in**3)        
!        
      integer(kind=ikd), target, allocatable, save :: i_gap(:)
      integer(kind=ikd), pointer, save :: &
!
       idxexp(:), &  !  index of file containing quantities for each cond in fuel rod stack                  
       igprgn(:), &  !  region number corresponding to the gap           
       istkc1(:)     !  internal number for first cond in the stack      
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for stack conductor and related heat conduction arrays
!
!  All of the following real and integer pointer arrays are Rank (1:stk_cnd)
!  Space is allocated and pointers associations are made in subroutine allok8_gap_conductance
!
      real(kind=rkd), pointer, save :: &
        cdexp(:), &  !  clad diameteral elastic expansion (ft)           
        fdexp(:), &  !  fuel diameteral elastic expansion (ft)           
        gapth(:), &  !  current gap width (ft)                           
        gpcon(:), &  !  gap conductance (btu/sec-ft**2-f)                
         pint(:)     !  fuel-clad interfacial pressure (lbf/ft**2)       
!        
      integer(kind=ikd), pointer, save :: &
       ocondn(:)     !  conductor number for this axial node             
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_gap
!
!  Subroutine to allocate space and set pointers for gap related arrays.
!    Note: stk_cnd must be defined before allok8_gap_conductance is called.
!
!     allok8_gap_conductance is called from subroutine ingap
!
      use kind_specs 
      use restart_block_list
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_gap)) return
!
      if (stk_cnd .le. 0) then
         write (6, 110) stk_cnd
         call fail
         return
      endif
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_gap, i2,  caexp, r_gap)
         call r_pntr (allocatd, num_gap, i2,  fathx, r_gap)
         call r_pntr (allocatd, num_gap, i2, ftdens, r_gap)
         call r_pntr (allocatd, num_gap, i2, gasmol, r_gap)
         call r_pntr (allocatd, num_gap, i2, gsfrac, r_gap)
         call r_pntr (allocatd, num_gap, i2,   pgas, r_gap)
         call r_pntr (allocatd, num_gap, i2,  roufc, r_gap)
         call r_pntr (allocatd, num_gap, i2,  rouff, r_gap)
         call r_pntr (allocatd, num_gap, i2,  vplen, r_gap)
         len    = i2
         call r_pntr (allocatd, stk_cnd, i2,  cdexp, r_gap)
         call r_pntr (allocatd, stk_cnd, i2,  fdexp, r_gap)
         call r_pntr (allocatd, stk_cnd, i2,  gapth, r_gap)
         call r_pntr (allocatd, stk_cnd, i2,  gpcon, r_gap)         
         call r_pntr (allocatd, stk_cnd, i2,   pint, r_gap)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rgap   = i2
!
            allocate (r_gap(siz_rgap), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rgap, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_gap
            block  = 'r_gap'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rgap, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_gap      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_gap, i2, idxexp, i_gap)
         call i_pntr (allocatd, num_gap, i2, igprgn, i_gap)         
         call i_pntr (allocatd, num_gap, i2, istkc1, i_gap)

!
         len    = i2
         call i_pntr (allocatd, stk_cnd, i2, ocondn, i_gap)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_igap   = i2
!
            allocate (i_gap(siz_igap), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_igap, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_gap
            block  = 'i_gap'
            call build_restart_block_list (block, rdum, idum, ldum, siz_igap, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_gap      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' gap arrays (allok8_gap_conductance).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for dynamic gap conductance model arrays (allok8_gap_conductance).'/ &
          10x,'Number of conductors in the gap model stack (stk_cnd) = ',i6,', must be > 0 - programming error') 
!          
      end subroutine allok8_gap
!
      end module dynamic_gap
