!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for nonconducting heat exchanger related arrays
!
!  This module also includes the following subroutines
!
!     allok8_htx   
!     allok8_htxtbl 
!
!  ===============================================================================================================================
!
      module htx
!
      use kind_specs
!
      implicit none      
!
      integer(kind=ikd), save :: &
          num_htx, &  !  number of nonconducting heat exchangers
         siz_rhtx, &  !  size of the array allocated for the real htx pointer arrays
         siz_ihtx     !  size of the array allocated for the integer htx pointer arrays
!
      integer(kind=ikd), target, allocatable, save :: i_htx(:)
!
!  The following integer and real pointer arrays are Rank (1:num_htx)
!  Space is allocated and pointers associations are made in subroutine allok8_htx 
!
      integer(kind=ikd), pointer, save :: &
       iheqst(:), &  !  flow network number to which the heat exchanger belongs                                          
          ihq(:), &  !  last position in htxtbl                          
         ihtx(:), &  !  number of table points for ihtype = 2, 3 or 4
                     !  control block id for ihtype = 6 or 7 -- otherwise unused
       ihtype(:), &  !  flag for type of non-conducting heat exchanger   
       indhtx(:), &  !  index for time-heat removal rate table           
       ithtxq(:), &  !  trip number controlling heat exchanger           
         jvol(:)     !  volume number (new)                                     
!
      real(kind=rkd), target, allocatable, save :: r_htx(:)
!
      real(kind=rkd), pointer, save :: &
          htq(:), &  !  heat removal rate                                
        htxco(:), &  !  heat exchanger heat transfer coefficient         
       htxcoi(:), &  !  initial heat transfer coefficient                
       powerf(:), &  !  initial fraction of power to be removed          
         qhtr(:), &  !  pressurizer heater power                         
        qprev(:), &  !  pressurizer heater power (previous time step)    
         qtau(:), &  !  pressurizer heater decay constant                
         quap(:), &  !  peak power before pressurizer heater trips off   
          thq(:), &  !  heat exchanger primary side temperature          
       timoff(:), &  !  time pressurizer heater trips off                
         tsec(:)     !  heat exchanger secondary side temperature       
!
!  ===============================================================================================================================
!  Specifications and space allocations for nonconducting heat exchanger table related arrays
!  ===============================================================================================================================
!
      integer(kind=ikd), save :: &
          num_htbl, &  !  number of pairs of htx data points in htx table
         siz_rhtbl     !  size of the array allocated for the real htx table pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_htbl(:)
!
!  The following real pointer arrays are Rank (1:2*num_htbl) where 2 * num_htbl is the sum of the 
!       number of points in all htx tables.
!  Space is allocated and pointers associations are made in subroutine allok8_htxtbl (2nd call)
!
      real(kind=rkd), pointer, save :: &
       htxtbl(:)    !  time / normalized power pair
!       
      contains
!
!  ===============================================================================================================================
      subroutine allok8_htx  
!
!  Subroutine to allocate space and set pointers for nonconducting heat exchanger related arrays
!     allok8_heat_exchangers is called from subroutine inhtx 
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status, point_sum
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_htx)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      if (num_htx .le. 0) then
         write (6, 110) num_htx
         call fail
         return
      endif
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_htx, i2, iheqst, i_htx)
         call i_pntr (allocatd, num_htx, i2,    ihq, i_htx)
         call i_pntr (allocatd, num_htx, i2,   ihtx, i_htx)
         call i_pntr (allocatd, num_htx, i2, ihtype, i_htx)
         call i_pntr (allocatd, num_htx, i2, indhtx, i_htx)
         call i_pntr (allocatd, num_htx, i2, ithtxq, i_htx)
         call i_pntr (allocatd, num_htx, i2,   jvol, i_htx)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ihtx   = i2
!
            allocate (i_htx(siz_ihtx), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ihtx, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_htx
            block  = 'i_htx'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ihtx, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_htx      = 0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Loop over all nonconduction heat exchanger data tables to allocate space for all table arrays.  Note: table arrays 
!  for all data tables reside in a single array htxtbl.  The beginning index for htxtbl for table 1 is indhtx(1) = 1.  For 
!  the second table indhtx(2) = (indhtx(1) + 2*|ihtx(1)|). This pattern continues for all other tables. 
!
      point_sum = 0
      len       = 0
! 
!      do l = 1, num_htx
!         if (ihtype(l).ge.2 .and. ihtype(l).le.4) then
!            len    = len + 1
!            point_sum = point_sum + abs (ihtx(l))
!         endif
!      enddo
!
!      if (len.gt.0 .and. point_sum.le.0) then
!         write (6,120) point_sum
!         call fail
!         return
!      endif
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_htx, i2,    htq, r_htx)
         call r_pntr (allocatd, num_htx, i2,  htxco, r_htx)
         call r_pntr (allocatd, num_htx, i2, htxcoi, r_htx)
         call r_pntr (allocatd, num_htx, i2, powerf, r_htx)
         call r_pntr (allocatd, num_htx, i2,   qhtr, r_htx)
         call r_pntr (allocatd, num_htx, i2,  qprev, r_htx)
         call r_pntr (allocatd, num_htx, i2,   qtau, r_htx)
         call r_pntr (allocatd, num_htx, i2,   quap, r_htx)
         call r_pntr (allocatd, num_htx, i2,    thq, r_htx)
         call r_pntr (allocatd, num_htx, i2, timoff, r_htx)
         call r_pntr (allocatd, num_htx, i2,   tsec, r_htx)
!
!  Add real table array after other real arrays
!
!         len    = i2
!
!         if (point_sum .gt. 0) then
!            call r_pntr (allocatd, 2*point_sum, i2, htxtbl, r_htx)
!         endif
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rhtx   = i2
!
            allocate (r_htx(siz_rhtx), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rhtx, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_htx
            block  = 'r_htx'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rhtx, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_htx      = 0.0d0
         endif
      enddo
!
!
!  Initialize pointer and memory indices for general data tables
!
!      i2    = 1
! 
!      do l = 1, num_htx
!         if (ihtype(l).ge.2 .and. ihtype(l).le.4) then
!            indhtx(l) = i2
!            ihq(l)    = 1
!            i2        = i2 + 2*abs (ihtx(l))
!         endif
!      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' nonconducting heat exchanger arrays (allok8_htx).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for nonconducting heat exchanger related arrays (allok8_htx). '/ &
          10x,'Number of heat exchagers (num_htx) = ',i3, ', must be > 0.')
  120 format (' ******** Error allocating space for nonconducting heat exchanger table arrays.'/ &
          10x,'Sum of number of points for all general data tables = ',i6,', must be > 0 - programming error - ' &
              '(allok8_htx)') 
!              
      end subroutine allok8_htx
!
!  ===============================================================================================================================
!
      subroutine allok8_htxtbl  
!
!  Subroutine to allocate space and set pointers for nonconducting heat exchanger table related arrays
!     allok8_htxtbl is called from subroutine inhtx 
! 
      use kind_specs
      use restart_block_list
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
!
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, len, i2, status, point_sum
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_htbl)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
      if (num_htbl .le. 0) then
         write (6, 110) num_htbl
         call fail
         return
      endif
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, 2*num_htbl, i2, htxtbl, r_htbl)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_rhtbl   = i2
!
            allocate (r_htbl(siz_rhtbl), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rhtbl, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_htbl
            block  = 'r_htbl'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rhtbl, 1, 2)
!
!     initialize integer arrays to 0
!
         else
            r_htbl      = 0.0d0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' nonconducting heat exchanger table arrays (allok8_htxtbl).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for nonconducting heat exchanger table related arrays (allok8_htxtbl). '/ &
          10x,'Number of table entries (num_htbl) = ',i3, ', must be > 0.')
!              
      end subroutine allok8_htxtbl
!      
      end module htx
