!$Header$
!$Copyright$
!
!  ==============================================================================================================================
!  Specifications and procedures used to create minor edit search list and procedures for locating minor edit requests and 
!  returning values
!
!  This module also includes the following subroutines
!
!     build_minor_edit_list
!     add_r_me
!     add_r2_me
!     add_i_me
!     add_i2_me
!     add_l_me
!     add_subreg_me
!     add_blk_entry
!     me_search
!     me_value
!     add_me_list
!
!  ==============================================================================================================================
!
      module minor_edit_search
!
      use kind_specs
!
!  Derived type specification used to define minor edit search list
!
      type me_search_list
         real(kind=rkd),    pointer :: r_edit_pntr(:),    &  ! pointer for real minor edit variable
                                       r2_edit_pntr(:,:)       ! pointer for 2d real minor edit variable
         
         integer(kind=ikd), pointer :: i_edit_pntr(:),    &  ! pointer for integer minor edit variable
                                       i2_edit_pntr(:,:), &  ! pointer for 2d integer minor edit variable
                                       me_offset_pntr(:), &  ! pointer to offset of first element in minor edit pointer array
                                       me_nod_pntr(:)        ! pointer to number of sub-region nodes
         logical(kind=lkd), pointer :: l_edit_pntr(:),    &  ! pointer for logical minor edit variable
                                       l2_edit_pntr(:,:)     ! pointer for 2d logical minor edit variable
         integer(kind=ikd)          :: pntr_type, &          ! pointer type flag -- 1 = real / 2 = integer / 3 = logical
                                       me_blk, &             ! block number minor edit belongs to
                                       me_units              ! integer flag identifying minor edit units
         character(kind=ckd,len=8)  :: me_flag               ! minor edit request flag
         character(kind=ckd,len=12) :: me_title              ! minor edit title string
  
      end type me_search_list
!
!  Derived type specification used to define data block characteristics, e.g., volume block, junction block, etc.
!
      type me_blk_list
         integer(kind=ikd), pointer :: me_reg_pntr(:)    ! pointer to region number for block
         integer(kind=ikd)          :: me_reg_test       ! region test flag for block 
                                                         ! >0 - use sequential region test 1 <= region <= me_reg_test
                                                         ! =0 - no region test
                                                         ! <0 - region must match valid me_reg_pntr
         character(kind=ckd,len=8)  :: blk_hdr           ! block label string -- i.e. VOL  999, SYSTEM, etc.
      end type me_blk_list
!
!  Minor edit variable and pointer search list
!
      type (me_search_list), target, allocatable, save :: me_list(:)
      type (me_blk_list),    target, allocatable, save :: blk_list(:)
      integer(kind=ikd), save :: siz_me_list, siz_blk_list, block_num
      logical(kind=lkd) allok8d
!
      contains
!
!================================================================================================================================
!
      subroutine build_minor_edit_list
!
!  Subroutine to allocate space and build list of valid minor edit flags and pointers for the associated 
!  edit variables.
! 
      implicit none
!
      integer(kind=ikd) l, n, status, reg_test
!
      if (allocated (me_list)) return
!
      do l = 1, 2
         allok8d = l.eq.2
         n        = 0
         block_num  = 0
!
         call add_me_list(n, block_num, allok8d)
!
!      allocate space for minor edit block characteristics and search list
!
         if (.not.allok8d) then
            siz_me_list  = n

            allocate (me_list(siz_me_list), stat=status)
!
!        check status of allocate
!
            if (status .ne. 0) then
               write (6,100) siz_me_list, status
               call fail
            end if
!
            siz_blk_list  = block_num
!
            allocate (blk_list(siz_blk_list), stat=status)
!
!        check status of allocate
!
            if (status .ne. 0) then
               write (6,110) siz_blk_list, status
               call fail
            end if
         end if
      end do
!  
      return
!
  100 format (/'Error allocating space for list of valid minor edit variables.  Length requested = ',i8/ &
               'Status flag returned from allocate = ',i3)
  110 format (/'Error allocating space for minor edit block characteristics.  Length requested = ',i8/ &
               'Status flag returned from allocate = ',i3)
      end subroutine build_minor_edit_list
!
!  ==============================================================================================================================
!
      subroutine add_r_me (len, r_me_pntr, flag, title, iunits)
!
!  Subroutine to determine space requirements and make entries into array me_list for valid real minor edit request flags
!
!     len       = cummulative length of target array                                               input/output
!     r_me_pntr = pointer to minor edit variable                                                   input
!     flag      = char string used to request a minor edit                                         input
!     title     = minor edit title string                                                          input
!     iunits    = integer index for minor edit units                                               input
!
      use kind_specs
!
      implicit none
!      
      integer(kind=ikd)  len, iunits
      real(kind=rkd),    pointer :: r_me_pntr(:)
      character(kind=ckd,len=8) flag
      character(kind=ckd,len=12) title
!
      len   = len + 1
!
      if (allok8d) then
         me_list(len) % r_edit_pntr => r_me_pntr 
         me_list(len) % pntr_type   =  1
         me_list(len) % me_blk      =  block_num
         me_list(len) % me_units    =  iunits
         me_list(len) % me_flag     =  flag 
         me_list(len) % me_title    =  title
      end if
!
      return
      end subroutine add_r_me
!
!  ==============================================================================================================================
!
      subroutine add_r2_me (len, r2_me_pntr, flag, title, iunits, ireg)
!
!  Subroutine to determine space requirements and make entries into array me_list for valid real minor edit request flags
!
!     len        = cummulative length of target array                                               input/output
!     r2_me_pntr = 2d pointer to minor edit variable                                                input
!     flag       = char string used to request a minor edit                                         input
!     title      = minor edit title string                                                          input
!     iunits     = integer index for minor edit units                                               input
!     ireg       = index for left or right surface                                                  input
!
      use kind_specs
!
      implicit none
!      
      integer(kind=ikd)  len, ireg, iunits
      real(kind=rkd),    pointer :: r2_me_pntr(:,:)
      character(kind=ckd,len=8) flag
      character(kind=ckd,len=12) title
!
      len   = len + 1
!
      if (allok8d) then
         me_list(len) % r2_edit_pntr => r2_me_pntr 
         me_list(len) % pntr_type   =  3 + ireg
         me_list(len) % me_blk      =  block_num
         me_list(len) % me_units    =  iunits
         me_list(len) % me_flag     =  flag 
         me_list(len) % me_title    =  title
      end if
!
      return
      end subroutine add_r2_me
!
!  ==============================================================================================================================
!
      subroutine add_i_me (len, i_me_pntr, flag, title, iunits)
!
!  Subroutine to determine space requirements and make entries into array me_list for valid integer minor edit request flags
!
!     len       = cummulative length of target array                                               input/output
!     i_me_pntr = pointer to minor edit variable                                                   input
!     flag      = char string used to request a minor edit                                         input
!     title     = minor edit title string                                                          input
!     iunits    = integer index for minor edit units                                               input
!
      use kind_specs
!
      implicit none
!
      integer(kind=ikd)  len, iunits
      integer(kind=ikd), pointer :: i_me_pntr(:)
      character(kind=ckd,len=8) flag
      character(kind=ckd,len=12) title
!
      len   = len + 1
!
      if (allok8d) then
         me_list(len) % i_edit_pntr => i_me_pntr 
         me_list(len) % pntr_type   =  2
         me_list(len) % me_blk      =  block_num
         me_list(len) % me_units    =  iunits
         me_list(len) % me_flag     =  flag 
         me_list(len) % me_title    =  title
      end if
!
      return
      end subroutine add_i_me
!
!  ==============================================================================================================================
!
      subroutine add_i2_me (len, i2_me_pntr, flag, title, iunits, ireg)
!
!  Subroutine to determine space requirements and make entries into array me_list for valid integer minor edit request flags
!
!     len        = cummulative length of target array                                               input/output
!     i2_me_pntr = 2d pointer to minor edit variable                                                input
!     flag       = char string used to request a minor edit                                         input
!     title      = minor edit title string                                                          input
!     iunits     = integer index for minor edit units                                               input
!     ireg       = index for left or right surface                                                  input
!
      use kind_specs
!
      implicit none
!
      integer(kind=ikd)  len, iunits, ireg
      integer(kind=ikd), pointer :: i2_me_pntr(:,:)
      character(kind=ckd,len=8) flag
      character(kind=ckd,len=12) title
!
      len   = len + 1
!
      if (allok8d) then
         me_list(len) % i2_edit_pntr => i2_me_pntr 
         me_list(len) % pntr_type   =  5 + ireg
         me_list(len) % me_blk      =  block_num
         me_list(len) % me_units    =  iunits
         me_list(len) % me_flag     =  flag 
         me_list(len) % me_title    =  title
      end if
!
      return
      end subroutine add_i2_me
!
!  ==============================================================================================================================
!
      subroutine add_l_me (len, l_me_pntr, flag, title, iunits)
!
!  Subroutine to determine space requirements and make entries into array me_list for valid logical minor edit request flags
!
!     len       = cummulative length of target array                                               input/output
!     l_me_pntr = pointer to minor edit variable                                                   input
!     flag      = char string used to request a minor edit                                         input
!     title     = minor edit title string                                                          input
!     iunits    = integer index for minor edit units                                               input
!
      use kind_specs
!
      implicit none
!
      integer(kind=ikd)  len, iunits
      logical(kind=lkd), pointer :: l_me_pntr(:)
      character(kind=ckd,len=8) flag
      character(kind=ckd,len=12) title
!
      len   = len + 1
!
      if (allok8d) then
         me_list(len) % l_edit_pntr => l_me_pntr 
         me_list(len) % pntr_type   =  3
         me_list(len) % me_blk      =  block_num
         me_list(len) % me_units    =  iunits
         me_list(len) % me_flag     =  flag 
         me_list(len) % me_title    =  title
      end if
!
      return
      end subroutine add_l_me
!
!  ==============================================================================================================================
!
      subroutine add_l2_me (len, l2_me_pntr, flag, title, iunits, ireg)
!
!  Subroutine to determine space requirements and make entries into array me_list for valid logical minor edit request flags
!
!     len        = cummulative length of target array                                               input/output
!     l2_me_pntr = 2d pointer to minor edit variable                                                input
!     flag       = char string used to request a minor edit                                         input
!     title      = minor edit title string                                                          input
!     iunits     = integer index for minor edit units                                               input
!     ireg       = index for left or right 
!
      use kind_specs
!
      implicit none
!
      integer(kind=ikd)  len, iunits, ireg
      logical(kind=lkd), pointer :: l2_me_pntr(:,:)
      character(kind=ckd,len=8) flag
      character(kind=ckd,len=12) title
!
      len   = len + 1
!
      if (allok8d) then
         me_list(len) % l2_edit_pntr => l2_me_pntr 
         me_list(len) % pntr_type    =  7 + ireg
         me_list(len) % me_blk       =  block_num
         me_list(len) % me_units     =  iunits
         me_list(len) % me_flag      =  flag 
         me_list(len) % me_title     =  title
      end if
!
      return
      end subroutine add_l2_me
!
!  ==============================================================================================================================
!
      subroutine add_subreg_me (len, r_me_pntr, off_pntr, nod_pntr, flag, title, iunits)
!
!  Subroutine to determine space requirements and make entries into array me_list for valid real subregion minor edit 
!  request flags
!
!     len       = cummulative length of target array                                               input/output
!     r_me_pntr = pointer to minor edit variable                                                   input
!     off_pntr  = offset pointer - gives offset to first element in minor edit array               input
!     nod_pntr  = pointer to number of elements in minor edit array                                input
!     flag      = char string used to request a minor edit                                         input
!     title     = minor edit title string                                                          input
!     iunits    = integer index for minor edit units                                               input
!
      use kind_specs
!
      implicit none
!
      integer(kind=ikd)  len, iunits
      real(kind=rkd),    pointer :: r_me_pntr(:)
      integer,    pointer :: off_pntr(:)
      integer,    pointer :: nod_pntr(:)
      character(kind=ckd,len=8) flag
      character(kind=ckd,len=12) title
!
      len   = len + 1
!
      if (allok8d) then
         me_list(len) % r_edit_pntr    => r_me_pntr 
         me_list(len) % pntr_type      =  1
         me_list(len) % me_blk         =  block_num
         me_list(len) % me_units       =  iunits
         me_list(len) % me_flag        =  flag 
         me_list(len) % me_title       =  title
         me_list(len) % me_offset_pntr => off_pntr
         me_list(len) % me_nod_pntr    => nod_pntr
      end if
!
      return
      end subroutine add_subreg_me
!
!  ==============================================================================================================================
!
      subroutine add_blk_entry (len, reg_pntr, reg_flag, hdr)
!
!  Subroutine to determine space requirements and make entries into array blk_list for each block of minor 
!  edit request flags
!
!     len       = cummulative length of target array                                               input/output
!     reg_pntr  = pointer to minor edit region test array                                          input
!     reg_flag  = region test flag                                                                 input
!     hdr       = minor edit block header string                                                   input
!
      use kind_specs
!
      implicit none
!
      integer(kind=ikd)  len, reg_flag
      integer(kind=ikd),  pointer :: reg_pntr(:)
      character(kind=ckd,len=8) hdr
!
      len   = len + 1
!
      if (allok8d) then
         blk_list(len) % me_reg_pntr => reg_pntr 
         blk_list(len) % me_reg_test =  reg_flag
         blk_list(len) % blk_hdr     =  hdr
      end if
!
      return
      end subroutine add_blk_entry
!
!  ==============================================================================================================================
!
      subroutine me_search (req_flg, region, faild, i, reg_off, err_off)
!
!  Subroutine to set pointer for valid minor edit requests 
!
!     req_flg      = minor edit request character string (any case on input)                       input/output
!                    (will be upper case on exit)
!     region       = region number for minor edit request                                          input
!     faild        = error flag                                                                    output
!     i            = index to minor edit description                                               output
!     reg_off      = pointer offset for requested comp                                             output
!     err_off      = turn off error checkin flag                                                   input
!
      use kind_specs
!
      implicit none
!
      integer(kind=ikd), pointer :: n_pntr(:), off_pntr(:)
      integer(kind=ikd)  i, j, blk, reg_test, reg_off, region, ncomp, nodes
      logical(kind=lkd)  faild, subreg, err_off
      character(kind=ckd,len=4) flag, req_flg, subr_flg
!     
!  Convert any lower case characters in minor edit request flag to upper case for search.  This makes minor edit request 
!  flags case insensitive.
!
      do i = 1, 4
         if (lge (req_flg(i:i), 'a') .and. lle (req_flg(i:i), 'z')) then
            j         = ichar(req_flg(i:i))
            flag(i:i) = char (j-32)
         else
            flag(i:i) = req_flg(i:i)
         end if
      end do
!
      req_flg = flag      
!
!  Check for subregion request - bottom 1, 2 or 3 characters must be integers (0 - 9).  If not 
!  integer characters, the request is not a subregion request.
!
      i        = 4
      subr_flg = '    '
!
      do while (lge (flag(i:i), '0') .and. lle (flag(i:i), '9')) 
         i      = i - 1   
      end do
!
      if (i .lt. 4) then 
         subr_flg(i+1:4) = flag(i+1:4)
         read (subr_flg, '(i4)') nodes
         subr_flg        = flag(1:i)//'    '
         flag            = subr_flg
         subreg          = .true.
      else
         subreg          = .false.
      end if
!
      i      = 1
      faild  = .false.
!
!  Sequentially search through all valid minor edit variables
!
      do while (i .le. siz_me_list)    
         if (flag .eq. me_list(i)%me_flag) then
            blk      = me_list(i)%me_blk
            reg_test = blk_list(blk)%me_reg_test
!
!     Minor edit flag found - now check for valid region
!
!        Case 1 - Check region number against sequnential range (1 to n)
!
            if (reg_test .gt. 0) then
               if (region.ge.1 .and. region.le.reg_test) then
                  reg_off = region
                  go to 10
               else
                  if ( .not.err_off ) then
                     write (6, 102) req_flg, region
                     faild  = .true.
                  end if
                  go to 10                     
               end if
!
!        Case 2 - No region check
!
            else if (reg_test .eq. 0) then
               reg_off = 1
               go to 10
!
!        Case 3 - Check region number against actual component number
!
            else
               ncomp   = abs (reg_test)
               do j = 1, ncomp
                  if (region .eq. blk_list(blk)%me_reg_pntr(j)) then
                     reg_off = j
                     go to 10
                  end if
               end do
!
               if ( .not.err_off ) then
                  write (6, 102) req_flg, region
                  faild  = .true.
               end if
               go to 10
            end if
!
!     Increment index for minor edit variable list
!
         else
            i      = i + 1
         end if
      end do
!
!  Minor edit flag not found in list of valid minor edit variables
!
      if ( .not.err_off ) then
         write (6,101) req_flg, region
         faild  = .true.
      end if
!
!  Adjust pointer offset if a subregion minor edit request
!
   10 continue
      if (subreg) then
         n_pntr   => me_list(i)%me_nod_pntr
         off_pntr => me_list(i)%me_offset_pntr
!
         if (nodes.ge.1 .and. nodes.le.n_pntr(reg_off)) then
            reg_off = off_pntr(reg_off) + nodes - 1
         else
            if ( .not.err_off ) then
               write (6, 103) req_flg, region
               faild  = .true.
            end if
         end if
      end if
!
      return
!
  101 format (/' ******** Error - Illegal minor edit request flag - ', a4,' - associated region number = ',i5)
  102 format (/' ******** Error - Invalid region number for minor edit request ',a4,1x,i5)
  103 format (/' ******** Error - Invalid subregion number for minor edit request ',a4,1x,i5)
      end subroutine me_search
!
!  ==============================================================================================================================
!
      function me_value (me_idx, reg_off)
!
!  Function to return the value of minor edit request as a real value
!
!     me_idx    = index of minor edit in me_list                                                   input
!     reg_off   = region offset - index for pointer array                                          input      
!
      use kind_specs
!
      implicit none
!
      real(kind=rkd)     me_value
      integer(kind=ikd)  icase, me_idx, reg_off
      real(kind=rkd)     r_var_pntr
      integer(kind=ikd)  i_var_pntr
      logical(kind=lkd)  l_var_pntr
!
      icase  = me_list(me_idx) % pntr_type
!
      select case (icase)
      case(1)
         r_var_pntr = me_list(me_idx) % r_edit_pntr(reg_off)
         me_value   = r_var_pntr
      case(2)
         i_var_pntr = me_list(me_idx) % i_edit_pntr(reg_off)
         me_value   = float (i_var_pntr)
      case(3)
         l_var_pntr = me_list(me_idx) % l_edit_pntr(reg_off)
         if (l_var_pntr) then
            me_value   = 1.0d0
         else
            me_value   = 0.0d0
         end if
      case(4)
         r_var_pntr = me_list(me_idx) % r2_edit_pntr(reg_off,1)
         me_value   = r_var_pntr
      case(5)
         r_var_pntr = me_list(me_idx) % r2_edit_pntr(reg_off,2)
         me_value   = r_var_pntr      
      case(6)
         i_var_pntr = me_list(me_idx) % i2_edit_pntr(reg_off,1)
         me_value   = float (i_var_pntr)      
      case(7)         
         i_var_pntr = me_list(me_idx) % i2_edit_pntr(reg_off,2)
         me_value   = float (i_var_pntr)            
      case(8)
         l_var_pntr = me_list(me_idx) % l2_edit_pntr(reg_off,1)      
      case(9)         
         l_var_pntr = me_list(me_idx) % l2_edit_pntr(reg_off,2)            
      end select
!
      if (icase == 8 .or. icase ==9) then
         if (l_var_pntr) then
            me_value   = 1.0d0
         else
            me_value   = 0.0d0
         end if
      end if                        
!
      return
      end function me_value
!
!================================================================================================================================
!
      subroutine add_me_list(n, blk_num, allocatd)
!
!  Subroutine to allocate space and build list of valid minor edit flags and pointers for the associated 
!  edit variables
!
      use kind_specs
      use problem_dimensions
      use volumes
      use junctions
      use system
      use conductors
      use separators
      use time_steps
      use trips
      use bubble_rise
      use system
      use core
      use valves
      use centrifugal_pumps
      use accumulators
      use turbines
      use point_kinetics
      use scram
      use dynamic_gap
      use htx
      use two_reg_neq
      use control_system
      use auxiliary_dnb
      use general_transport
!      
      implicit none   
!     
!  Dummy variables
!  
      integer(kind=ikd) :: n, blk_num
      logical(kind=lkd) :: allocatd
      intent(inout) n, blk_num, allocatd
!
!  Local variables
!
      integer(kind=ikd) :: reg_test, status
      integer(kind=ikd), pointer, dimension(:) :: idummy
!      
      allocate(idummy(1),stat=status)
      
!      
!  Add volume related minor edit flags
!--------------------------------------------------------------------------------------------------------------------------------
!
      reg_test =  -num_vol  ! use olvoln to do region test
!
      call add_blk_entry (blk_num, olvoln, reg_test, 'VOL     ')
!
      call add_r_me (n, p      , 'PRES    ', 'AVG. PRESS. ',  4)
      call add_r_me (n, temp   , 'TEMP    ', ' AVG. TEMP. ',  1)
      call add_r_me (n, satp   , 'SATP    ', 'SAT. PRESS. ',  4)
      call add_r_me (n, satt   , 'SATT    ', ' SAT. TEMP. ',  1)
      call add_r_me (n, satvf  , 'STVF    ', '  SAT. VF   ',  6)
      call add_r_me (n, satvg  , 'STVG    ', '  SAT. VG   ',  6)
      call add_r_me (n, sathf  , 'STHF    ', '  SAT. HF   ',  7)
      call add_r_me (n, sathg  , 'STHG    ', '  SAT. HG   ',  7)
      call add_r_me (n, hw     , 'HW**    ', 'AVG ENTHALPY',  7)
      call add_r_me (n, avex   , 'AVEX    ', 'AVG. QUALITY', 27)
      call add_r_me (n, thcong , 'GCON    ', 'AIR CONDUCTV', 27)
      call add_r_me (n, viscg  , 'GVIS    ', 'AIR VISCOSTY', 27)
      call add_r_me (n, xnc    , 'XNC*    ', ' NCG QUAITY ', 27)
      call add_r_me (n, presnc , 'PNC*    ', 'NCG PRESSURE',  4)
      call add_r_me (n, vaporv , 'VAPV    ', 'VAP SPEC VOL',  6)
      call add_r_me (n, hum    , 'HUM*    ', 'REL HUMIDITY', 27)
      call add_r_me (n, armass , 'AIRM    ', '  NCG MASS  ',  8)
      call add_r_me (n, dpdmf  , 'DPDM    ', '  DPDMF     ', 27)
      call add_r_me (n, dpdu   , 'DPDU    ', '  DPDU      ', 27)
      call add_r_me (n, dpdmv  , 'DPDV    ', '  DPDMV     ', 27)
      call add_r_me (n, dpdma  , 'DPDA    ', '  DPDMA     ', 27)
      call add_r_me (n, vaporh , 'VAPH    ', ' VAPOR ENTH ',  7)
      call add_r_me (n, nonch  , 'AIRH    ', 'NCG ENTHALPY',  7)
      call add_r_me (n, xvapor , 'XVAP    ', 'VAP. QUALITY', 27)
      call add_r_me (n, vnonc  , 'VNON    ', 'NCG SPEC VOL',  6)
      call add_r_me (n, fmass  , 'FMAS    ', ' TOTAL MASS ',  8)
      call add_r_me (n, gasm   , 'GASM    ', '  GAS MASS  ',  8)
      call add_r_me (n, aved   , 'AVED    ', 'AVG. DENSITY',  9)
      call add_r_me (n, u      , 'UTOT    ', 'TOT INT ENER', 10)
      call add_r_me (n, wvbar  , 'WVBR    ', ' AVG. FLOW  ', 11)
      call add_r_me (n, wq     , 'WQ**    ', 'POWR TO H2O ', 12)
      call add_i_me (n, olvoln , 'OLVN    ', 'OLVOLN      ', 27)
      call add_i_me (n, iqin   , 'IQIN    ', 'IQIN        ', 27)
      call add_r_me (n, qspl   , 'QSPL    ', 'ENRGY SOURCE', 13)
      call add_r_me (n, oldp   , 'OLDP    ', 'OLDP        ', 27)
      call add_i_me (n, ineq   , 'INEQ    ', 'INEQ        ', 27)
      call add_r_me (n, voidn  , 'NVOI    ', 'NEUTRON VOID', 27)
      call add_r_me (n, voidv  , 'VOIV    ', 'VOID FRACTN ', 27)
      call add_r_me (n, wgv    , 'WGV*    ', '  GAS FLOW  ', 11)
      call add_r_me (n, wlv    , 'WLV*    ', 'LIQUID FLOW ', 11)
      call add_r_me (n, vslpv  , 'VSLV    ', '  SLIP VEL. ', 14)
      call add_r_me (n, vgasv  , 'VGV*    ', '  GAS VEL.  ', 14)
      call add_i_me (n, iequat , 'IEQU    ', 'IEQUAT      ', 27)
      call add_r_me (n, zlqlev , 'LIQL    ', ' LIQUID LEV ', 15)
      call add_r_me (n, liqmas , 'LIQM    ', ' LIQUID MASS',  8)
      call add_r_me (n, liqvol , 'LIQV    ', ' LIQUID VOL.', 16)
      call add_r_me (n, vl     , 'VL**    ', 'LIQ SPEC VOL',  6)
      call add_r_me (n, vs     , 'VS**    ', 'VAP SPEC VOL',  6)
      call add_r_me (n, vs     , 'VS**    ', 'GAS SPEC VOL',  6)
      call add_r_me (n, pncol  , 'PNOL    ', 'OLD PNC  VOL', 27)
      call add_r_me (n, dpdmol , 'DPD1    ', 'OLD DPDM VOL', 27)
      call add_r_me (n, delmfo , 'DMF1    ', 'OLD DLMF VOL', 27)
      call add_r_me (n, dpdaol , 'DPD2    ', 'OLD DPDA VOL', 27)
      call add_r_me (n, delmao , 'DMA1    ', 'OLD DLMA VOL', 27)
      call add_r_me (n, dpduol , 'DPD3    ', 'OLD DPDU VOL', 27)
      call add_r_me (n, deluo  , 'DU1*    ', 'OLD DELU VOL', 27)
      call add_r_me (n, dpvmol , 'DPV1    ', 'OLD DPVM VOL', 27)
      call add_r_me (n, dpvaol , 'DPV2    ', 'OLD DPVA VOL', 27)
      call add_r_me (n, dpvuol , 'DPV3    ', 'OLD DPVU VOL', 27)
      call add_r_me (n, dhlmol , 'DHL1    ', 'OLD DHLM VOL', 27)
      call add_r_me (n, dhlaol , 'DHL2    ', 'OLD DHLA VOL', 27)
      call add_r_me (n, dhluol , 'DHL3    ', 'OLD DHLU VOL', 27)
      call add_r_me (n, dhvmol , 'DHV1    ', 'OLD DHVM VOL', 27)
      call add_r_me (n, dhvaol , 'DHV2    ', 'OLD DHVA VOL', 27)
      call add_r_me (n, dhvuol , 'DHV3    ', 'OLD DHVU VOL', 27)
      call add_r_me (n, dxncmo , 'DXN1    ', 'OLD DXNM VOL', 27)
      call add_r_me (n, dxncao , 'DXN2    ', 'OLD DXNA VOL', 27)
      call add_r_me (n, dxncuo , 'DXN3    ', 'OLD DXNU VOL', 27)
      call add_r_me (n, dxvmol , 'DXV1    ', 'OLD DXVM VOL', 27)
      call add_r_me (n, dxvaol , 'DXV2    ', 'OLD DXVA VOL', 27)
      call add_r_me (n, dxvuol , 'DXV3    ', 'OLD DXVU VOL', 27)
      call add_r_me (n, gash   , 'GASH    ', 'GAS ENTHALPY',  7)
      call add_r_me (n, liqh   , 'LIQH    ', 'LIQ ENTHALPY',  7)
!      
!     five equation minor edit parameters 
!
      if (neweqs(1) .gt. 0) then
         call add_r_me (n, mdotlg , 'MDOT    ', 'MASS TRN RAT', 11)
         call add_r_me (n, qspll  , 'QLIQ    ', 'LIQ HTNG RAT', 13)
         call add_r_me (n, templ  , 'TEML    ', 'LIQUID TEMP.',  1)
         call add_r_me (n, tempv  , 'TEMV    ', 'VAPOR TEMP. ',  1)
         call add_r_me (n, mdtdt  , 'MDDT    ', ' MDOT / TEMP', 11)
         call add_r_me (n, htcif  , 'HTC*    ', 'INTERFAC HTC',  2)
         call add_i_me (n, ifrv   , 'IFRV    ', 'FLOW REGIME ', 27)
         call add_r_me (n, areaif , 'AREA    ', 'INT AREA DEN', 31)
         call add_r_me (n, dprdto , 'DPDT    ', '            ', 27)
      end if
!
!  Add junction related minor edit flags
!--------------------------------------------------------------------------------------------------------------------------------
!
      reg_test      =  -num_jun  ! use oljunn to do region test
!
      call add_blk_entry (blk_num, oljunn, reg_test, 'JUN     ')
!
      call add_i2_me (n, iw    , 'IW1*    ', 'IW1         ', 27,1)
      call add_i2_me (n, iw    , 'IW2*    ', 'IW2         ', 27,2)
      call add_r_me  (n, wp    , 'WP**    ', '    FLOW    ', 11)
      call add_r_me  (n, dela  , 'DELA    ', 'ACCEL. PRES.',  4)
      call add_r_me  (n, dele  , 'DELE    ', 'ELEV. PRES. ',  4)
      call add_r_me  (n, delf  , 'DELF    ', 'FRICT. PRES.',  4)
      call add_r_me  (n, xp    , 'XP**    ', '  QUALITY   ', 27)
      call add_r_me  (n, hp    , 'HP**    ', '  ENTHALPY  ',  7)
      call add_i_me  (n, itp   , 'ICHK    ', ' CHOKE MODE ', 27)
      call add_r_me  (n, pumpp , 'PMPP    ', ' PUMP PRES. ',  4)
      call add_r_me  (n, delp  , 'DELP    ', ' VEL. HEAD  ',  4)
      call add_r_me  (n, spvj  , 'SPVJ    ', ' SPEC. VOL. ',  6)
      call add_r_me  (n, ajunt , 'AJNT    ', 'TIM DEP AREA', 17)
      call add_i_me  (n, oljunn, 'OLJN    ', 'OLJUNN      ', 27)
      call add_r_me  (n, hpflow, 'HFLO    ', 'FLOW. ENTH. ',  7)
      call add_r_me  (n, wold  , 'WOLD    ', '  OLD FLOW  ', 11)
      call add_r_me  (n, hpl   , 'HPL*    ', 'LIQ ENTHALPY',  7)
      call add_r_me  (n, hpg   , 'HPG*    ', 'GAS ENTHALPY',  7)
      call add_r_me  (n, spvjl , 'SPJL    ', 'LIQ SPEC VOL',  6)
      call add_r_me  (n, spvjg , 'SPJG    ', 'GAS SPEC VOL',  6)
      call add_r_me  (n, void  , 'VOIJ    ', 'VOID FRACTN.', 27)
      call add_r_me  (n, vslpj , 'VSLJ    ', ' SLIP VEL.  ', 14)
      call add_r_me  (n, vslpjo, 'VSLO    ', 'SLIPO VEL.  ', 14)
      call add_r_me  (n, wlj   , 'WLJ*    ', '  LIQ FLOW  ', 11)
      call add_r_me  (n, wgj   , 'WGJ*    ', '  GAS FLOW  ', 11)
      call add_r_me  (n, fjunf , 'FJNF    ', 'F. LOSS COEF', 27)
      call add_r_me  (n, fjunr , 'FJNR    ', 'R. LOSS COEF', 27)
      call add_r_me  (n, avedj , 'AVDJ    ', '  DENSITY   ',  9)
      call add_r_me  (n, conco , 'CONC    ', 'CONT. COEF. ', 27)
      call add_r_me  (n, vgasj , 'VGJ*    ', '  GAS VEL.  ', 14)
      call add_r_me  (n, slpcnv, 'SCNV    ', 'SLIP CONVCTN', 13)
      call add_r_me  (n, dvsdt , 'DVDT    ', '   DVS/DT   ', 28)
      call add_r_me  (n, esubk , 'ESBK    ', '   DW/DT    ',  8)
      call add_i_me  (n, ifrjed, 'IFRL    ', 'FLOW REGIME ', 27)
      call add_r_me  (n, pjun  , 'PJUN    ', '  PRESSURE  ',  4)
      call add_r_me  (n, slipr , 'SLPR    ', ' SLIP RATIO ', 27)
      call add_r_me  (n, tempj , 'TEMJ    ', 'TEMPERATURE ',  1)
      call add_r_me  (n, xpnc  , 'XPNC    ', 'JUN NCG QUAL', 27)
!
!  Add overal system related minor edit flags
!--------------------------------------------------------------------------------------------------------------------------------
!
      reg_test      =  0  ! no region test (olvoln not used - supplied as a dummy arguement)
!
      call add_blk_entry (blk_num, olvoln, reg_test, ' SYSTEM ')
!
      call add_l_me (n, nogo  , 'NOGO    ', 'PROB FAILED ', 27)
      call add_r_me (n, ae    , 'AE**    ', 'ENERGY ADDED', 10)
      call add_r_me (n, amass , 'AMAS    ', ' MASS ADDED ',  8)
      call add_r_me (n, bmassw, 'BMSW    ', ' MASS WATER ',  8)
      call add_r_me (n, bmassa, 'BMSA    ', '  MASS AIR  ',  8)
      call add_r_me (n, eb    , 'EB**    ', 'ENERGY BAL. ', 10)
      call add_r_me (n, fe    , 'FE**    ', 'FUEL ENERGY ', 10)
      call add_r_me (n, he    , 'HE**    ', 'ENRGY REMOVD', 10)
      call add_r_me (n, pnorm , 'PNRM    ', 'NORM. POWER ', 27)
      call add_r_me (n, qloss , 'QLOS    ', 'HT REMOVL RT', 12)
      call add_r_me (n, cpusec, 'CPUS    ', '  CPU TIME  ', 18)
      call add_r_me (n, cnstnt, 'CONS    ', '  CONSTANT  ', 27)
      call add_r_me (n, timex , 'TIMX    ', 'ELAPSED TIME', 18)
      call add_r_me (n, power , 'POWR    ', ' CORE POWER ', 19)
      call add_r_me (n, ptherm, 'PTHR    ', 'TOTAL POWER ', 12)
      call add_r_me (n, prompp, 'PRPT    ', 'PROMPT POWER', 27)
      call add_r_me (n, delayp, 'DCAY    ', 'DECAY POWER ', 27)
!
!-----------------------------------------------------------------------
!  Add separator related minor edit flags
!-----------------------------------------------------------------------
!
      reg_test      =  -1   !  use isev to do region test
!
      call add_blk_entry (blk_num, isev, reg_test, ' SEP    ')         
!       
      call add_r_me (n, covd  , 'COVD    ', 'DESIGN COVER', 27)        
      call add_r_me (n ,cund  , 'CUND    ', 'DESIGN CUNDR', 27)        
      call add_r_me (n, rfsep , 'RFSP    ', 'LIQ LAYR RAD', 15)        
      call add_r_me (n, somega, 'OMGA    ', 'ANG VELOCITY', 32)        
!
!-----------------------------------------------------------------------
!  Add heat conductor related minor edit flags
!--------------------------------------------------------------------------------------------------------------------------------
!
      reg_test      =  -num_cond  ! use olslbn for region test
!
      call add_blk_entry (blk_num, olslbn, reg_test, 'COND    ')
!
      if (nslb(1) .gt. 0) then
         call add_r_me  (n, se     , 'SE**    ', 'STORD ENERGY', 10)
         call add_r2_me (n, fchl   , 'FCHL    ', '  LEFT CHF  ', 21,1)
         call add_r2_me (n, fchl   , 'FCHR    ', ' RIGHT CHF  ', 21,2)
         call add_r2_me (n, htcl   , 'HTCL    ', '  LEFT HTC  ',  2,1)
         call add_r2_me (n, htcl   , 'HTCR    ', ' RIGHT HTC  ',  2,2)
         call add_r2_me (n, phil   , 'PHIL    ', 'L HEAT FLUX ', 21,1)
         call add_r2_me (n, phil   , 'PHIR    ', 'R HEAT FLUX ', 21,2)
         call add_r2_me (n, tl     , 'TL**    ', 'L SURF TEMP ',  1,1)
         call add_r2_me (n, tl     , 'TR**    ', 'R SURF TEMP ',  1,2)
         call add_r2_me (n, wqcl   , 'WQCL    ', 'L POW TO H2O', 12,1)
         call add_r2_me (n, wqcl   , 'WQCR    ', 'R POW TO H2O', 12,2)
         call add_i2_me (n, ihtl   , 'IHTL    ', ' L HT MODE  ', 27,1)
         call add_i2_me (n, ihtl   , 'IHTR    ', ' R HT MODE  ', 27,2)
         call add_i_me  (n, isco   , 'ISCO    ', 'ISCO        ', 27)
         call add_i_me  (n, olslbn , 'OSLB    ', 'OLSLBN      ', 27)
         call add_i2_me (n, ivslol , 'IVSL    ', 'IVSLOL      ', 27,1)
         call add_i2_me (n, ivslol , 'IVSR    ', 'IVSROL      ', 27,2)
         call add_r2_me (n, gl     , 'GL**    ', 'L MASS FLUX ',  5,1)
         call add_r2_me (n, gl     , 'GR**    ', 'R MASS FLUX ',  5,2)
         call add_r2_me (n, ahtl   , 'AHTL    ', 'AHTL        ', 27,1)
         call add_r2_me (n, ahtl   , 'AHTR    ', 'AHTR        ', 27,2)
         call add_r2_me (n, xlocl  , 'XLOL    ', 'L LOCAL QUAL', 27,1)
         call add_r2_me (n, xlocl  , 'XLOR    ', 'R LOCAL QUAL', 27,2)            
         call add_r2_me (n, tlocl  , 'TLOL    ', 'L LOCAL TEMP',  1,1)
         call add_r2_me (n, tlocl  , 'TLOR    ', 'R LOCAL TEMP',  1,2)            
         call add_r2_me (n, alphl  , 'ALPL    ', 'L. VOID FRAC', 27,1)
         call add_r2_me (n, alphl  , 'ALPR    ', 'R. VOID FRAC', 27,2)
         call add_r2_me (n, avedl  , 'AVDL    ', 'L. DENSITY  ',  9,1)
         call add_r2_me (n, avedl  , 'AVDR    ', 'R. DENSITY  ',  9,1)
         call add_r2_me (n, cpleft , 'CSPL    ', 'L. SP. HEAT ',  3,1)
         call add_r2_me (n, cpleft , 'CSPR    ', 'R. SP. HEAT ',  3,2)
!
!        Subregion minor edit variables
!
         call add_subreg_me (n, tp     , idxtp  , nnode , 'T       ', 'NODE TEMP   ',  1)
!
      end if 
!     
!  Add trip related minor edit flags
!--------------------------------------------------------------------------------------------------------------------------------
!
         reg_test      =  -1  ! use idtrp to do region test
!
         call add_blk_entry (blk_num, idtrp, reg_test, 'TRIP    ')
!
         call add_r_me (n, tsig  , 'TSIG    ', 'SIGNAL TIME ', 18)        
         call add_r_me (n, abx   , 'ABX*    ', 'ABX         ', 27)        
         call add_l_me (n, flog  , 'FLOG    ', 'FLOG        ', 27)
!
!  Add bubble rise related minor edit flags
!-----------------------------------------------------------------------
!                                                                       
         reg_test      =  -num_bub   ! use ibubol to do region test
!
         call add_blk_entry (blk_num, ibubol, reg_test, 'BUBL    ')         
!         
         call add_r_me (n, alph  , 'ALPH    ', 'BUBBLE GRAD.', 27)        
         call add_r_me (n, vbub  , 'VBUB    ', 'BUBBLE VEL. ', 14)        
         call add_r_me (n, zm    , 'MIXL    ', ' MIX. LEVEL ', 15)        
         call add_r_me (n, bubm  , 'BUBM    ', 'BUBBLE MASS ', 8 )        
         call add_r_me (n, mixv  , 'MIXV    ', 'MIX. VOLUME ', 16)        
         call add_r_me (n, liqv  , 'LIQV    ', ' LIQ VOLUME ', 16)        
         call add_r_me (n, mixq  , 'MIXQ    ', 'MIX QUALITY ', 27)        
         call add_r_me (n, vbnorm, 'VBNR    ', ' VBUB NORM  ', 27)        
         call add_r_me (n, dzmdt , 'DZDT    ', 'DMIXLDT     ', 27)        
         call add_i_me (n, ibubol, 'IBOL    ', 'IBUBOL      ', 27)     
!
!  Add valve related minor edit flags   
!-----------------------------------------------------------------------
!
         reg_test      =  -1   ! use ivalve to do region test
!
         call add_blk_entry (blk_num, ivalve, reg_test, 'VALV    ')         
!                                                                       
         call add_r_me (n, thdot , 'THDT    ', 'THDOT       ', 27)        
         call add_r_me (n, theta , 'THTA    ', 'THETA       ', 27)        
         call add_l_me (n, open  , 'OPEN    ', ' VALVE OPEN ', 27)    
!
!  Add pump related minor edit flags    
!-----------------------------------------------------------------------
!                                                                       
         reg_test      =  num_pump   ! use olpmpn to do region test
!
         call add_blk_entry (blk_num, olpmpn, reg_test, 'PUMP    ')         
!         
         call add_r_me (n, pomga , 'SPED    ', ' PUMP SPEED ', 20)        
         call add_r_me (n, torque, 'TORQ    ', ' HYD TORQUE ', 27)        
         call add_i_me (n, olpmpn, 'OLPN    ', 'OLPMPN      ', 27)     
         call add_i_me (n, ivpmpo, 'IVPM    ', 'IVPMPO      ', 27)     
         call add_r_me (n, pmtorq, 'PMTQ    ', ' MOTOR TORQ ', 27)        
         call add_r_me (n, pftorq, 'PFTQ    ', 'FRICTN TORQ ', 27)        
         call add_r_me (n, pmpowr, 'PMPW    ', ' PUMP POWR  ', 12)        
         call add_r_me (n, vrhoi , 'VROI    ', 'VRHOI       ', 27)        
         call add_r_me (n, torkmr, 'TQMR    ', 'TORKMR      ', 27)        
         call add_r_me (n, torkfr, 'TQFR    ', 'TORKFR      ', 27)        
         call add_r_me (n, pomgao, 'SPDO    ', 'POMGAO      ', 27)        
!
!  Add core conductor related minor edit flags
!-----------------------------------------------------------------------
!                                                                       
         reg_test      =  num_core   ! use islb to do region test
!
         call add_blk_entry (blk_num,   islb, reg_test, 'CORE    ')         
!         
         call add_i_me (n, islb  , 'ISLB    ', 'ISLB        ', 27)
         call add_r_me (n, fq    , 'CORQ    ', 'COND HTRATE ', 12)        
         call add_r_me (n, tm    , 'TAVG    ', ' AVG. TEMP. ', 1)        
         call add_r_me (n, ctr   , 'CTR*    ', 'MW REAC. OUT', 15)        
         call add_r_me (n, ctrin , 'CTRI    ', 'MW REAC. IN ', 15)        
         call add_r_me (n, qmwr  , 'QMWR    ', 'MW-R HT RATE', 12)        
         call add_r_me (n, mq    , 'MODQ    ', 'MOD. HT RATE', 12)        
         call add_r_me (n, qfrac , 'QFRA    ', 'POWER FRACTN', 27)        
         call add_r_me (n, spv0  , 'SPV0    ', 'SPV0        ', 27)        
         call add_r_me (n, ts    , 'TSUR    ', 'SURFACE TEMP',  1) 
         call add_r_me (n, hfuel , 'FENT    ', 'FUEL ENTHLPY', 33)
         call add_r_me (n, dhfuel, 'DFEN    ', 'DELTA FUEL H', 33)
!
!  Add accumulator related minor edit flags
!-----------------------------------------------------------------------
         reg_test      =  -1   ! use idvacc to do region test
!
         call add_blk_entry (blk_num, idvacc, reg_test, 'ACCM    ')         
!
         call add_r_me (n, tair  , 'TAIR    ', '  AIR TEMP  ',  1)        
         call add_r_me (n, excoef, 'EXPC    ', 'EXPANS. COEF', 27)        
         call add_r_me (n, acclev, 'WLEV    ', 'WATER LEVEL ', 15)        

!  Add turbine related minor edit flags
!-----------------------------------------------------------------------
!                                                                       
         reg_test      =  -1   ! use ntsha to do region test
!
         call add_blk_entry (blk_num,  ntsha, reg_test, 'TURB    ')         
!         
         call add_r_me (n, shrpm , 'SRPM    ', 'SHAFT SPEED ', 20)        
         call add_r_me (n, powout, 'POWO    ', 'SHAFT POWER ', 12)    
!
!-----------------------------------------------------------------------
!  Add kinetics constants related minor edit flags (old file 27)
!-----------------------------------------------------------------------
!                                                                       
         reg_test      =  0   ! no region test use idummy
!
         call add_blk_entry (blk_num, idummy, reg_test, 'REAC KIN')         
!
         call add_r_me (n, period, 'PERD    ', '   PERIOD   ', 18)        
         call add_r_me (n, preac , 'REAC    ', ' REACTIVITY ', 22)        
         call add_r_me (n, w     , 'W1**    ', 'INVRS PERIOD', 23)        
         call add_r_me (n, dt    , 'RKDT    ', 'KINETICS DT ', 18)        
         call add_r_me (n, ppow  , 'PPOW    ', 'PROMPT POWR ', 27)        
         call add_r_me (n, dpow  , 'DPOW    ', 'DECAY POWER ', 27)        
!
!-----------------------------------------------------------------------
!  Add reactivity table related minor edit flags (old file 28)
!-----------------------------------------------------------------------
!                                                                       
         reg_test      =  0   ! no region test use idummy
!
         call add_blk_entry (blk_num, idummy, reg_test, 'REAC KIN')         
!
         call add_r_me (n, rc    , 'RC**    ', 'CONTROL REAC', 22)        
         call add_r_me (n, rccal , 'RCIN    ', 'RCCAL       ', 27)        
!
!-----------------------------------------------------------------------
!  Add reactor kinetics related minor edit flags (old file 29 & 31)
!-----------------------------------------------------------------------
!                                                                       
         reg_test      =  0   ! no region test use idummy
!                
         call add_blk_entry (blk_num, idummy, reg_test, 'REAC KIN')         
!         
         call add_r_me (n, rw    , 'RW**    ', ' WATER TEMP ', 22)
         call add_r_me (n, rv    , 'RV**    ', '    VOID    ', 22)        
         call add_r_me (n, rf    , 'RF**    ', ' FUEL TEMP  ', 22)        
         call add_r_me (n, rd    , 'RD**    ', '  DOPPLER   ', 22)        
         call add_r_me (n, rhocal, 'ROIN    ', ' TOTAL REAC ', 22)        
         call add_r_me (n, rwcal , 'RWIN    ', ' WATER TEMP ', 22)        
         call add_r_me (n, rvcal , 'RVIN    ', '    VOID    ', 22)        
         call add_r_me (n, rfcal , 'RFIN    ', ' FUEL TEMP  ', 22)        
         call add_r_me (n, rdcal , 'RDIN    ', '  DOPPLER   ', 22)       
         call add_r_me (n, v1    , 'V1**    ', 'V1          ', 27)        
         call add_r_me (n, vw    , 'VW**    ', 'VW          ', 27)        
!         
!-----------------------------------------------------------------------
!  Add time step related minor edit flags
!-----------------------------------------------------------------------
!
         reg_test      =  0    ! no region test use i dummy
!
         call add_blk_entry (blk_num, idummy, reg_test, 'TIME STP')         
!
         call add_r_me (n, dtnext, 'DTNX    ', 'DTNEXT      ', 27)        
         call add_i_me (n, idmp  , 'IDMP    ', 'IDMP        ', 27)     
         call add_i_me (n, imaj  , 'IMAJ    ', 'IMAJ        ', 27)     
         call add_i_me (n, imin  , 'IMIN    ', 'IMIN        ', 27)     
         call add_i_me (n, iplt  , 'IPLT    ', 'IPLT        ', 27)     
         call add_i_me (n, nset  , 'NSET    ', 'NSET        ', 27)     
         call add_i_me (n, nstep , 'NSTP    ', '  STANDARD  ', 27)     
         call add_i_me (n, nttsp , 'NTTS    ', '   ACTUAL   ', 27)     
         call add_l_me (n, endstp, 'ENDS    ', 'ENDSTP      ', 27)     
         call add_l_me (n, reduc , 'RDUC    ', 'REDUC       ', 27)     
         call add_r_me (n, dtolds, 'DTOL    ', ' OLD VALUE  ', 18)        
         call add_r_me (n, dtavg , 'DTAV    ', '  AVERAGE   ', 18)        
!         
!-----------------------------------------------------------------------
!  Add dynamic gap conductance related minor edit flags
!-----------------------------------------------------------------------
!
         reg_test      =  -1    ! use igprgn to do region test
!
         call add_blk_entry (blk_num, igprgn, reg_test, ' GAP    ')         

         call add_r_me (n, pgas  , 'PGAS    ', 'FILL GAS PRS',  4)        
         call add_r_me (n, caexp , 'CAEX    ', 'CLAD AX EXP ', 15)        
         call add_r_me (n, fathx , 'FAEX    ', 'FUEL AX EXP ', 15)    
         call add_r_me (n, cdexp , 'CDEX    ', 'CLAD DIA EXP', 15)
         call add_r_me (n, fdexp , 'FDEX    ', 'FUEL DIA EXP', 15) 
         call add_r_me (n, gapth , 'GPTH    ', ' GAP WIDTH  ', 15)
         call add_r_me (n, pint  , 'PINT    ', 'CONTACT PRES',  4)
         call add_r_me (n, gpcon , 'GPCN    ', '  GAP COND  ',  2)
         call add_i_me (n, ocondn, 'OCND    ', '  COND NUM  ', 27)
!
!-----------------------------------------------------------------------
!  Add heat exchanger related minor edit flags
!-----------------------------------------------------------------------
!
         reg_test      =  num_htx   ! use jvol to do region test
!
         call add_blk_entry (blk_num,   jvol, reg_test, ' HTX    ')         
!         
         call add_i_me (n, jvol  , 'JVOL    ', 'JVOL        ', 27)     
         call add_r_me (n, thq   , 'THQ*    ', 'PRIMARY TEMP',  1)        
         call add_r_me (n, htq   , 'HTQ*    ', 'HT REMOVL RT', 12)        
         call add_r_me (n, tsec  , 'TSEC    ', 'SECNDRY TEMP',  1)        
         call add_r_me (n, htxco , 'HTXC    ', 'HT TRAN COEF',  2)        
         call add_r_me (n ,qprev , 'QPRV    ', 'QPREV       ', 27)        
         call add_r_me (n, quap  , 'QUAP    ', 'QUAP        ', 27)        
         call add_r_me (n, timoff, 'TMOF    ', 'TIMOFF      ', 27)        
!
!-----------------------------------------------------------------------
!  Add trip id related minor edit flags
!-----------------------------------------------------------------------
!
         reg_test      =  -num_tid   ! use oldtrp to do region test
!
         call add_blk_entry (blk_num, oldtrp, reg_test, 'TRIP    ')         
!         
         call add_r2_me (n, trpt  , 'TRPT    ', 'TIME SIG RCV', 18,1)        
         call add_r2_me (n, trpt  , 'RTPT    ', 'TIME SIG RCV', 18,2)        
         call add_l2_me (n, flag  , 'FLAG    ', 'FLAG        ', 27,1)              
         call add_l2_me (n, flag  , 'FLGR    ', 'FLAGR       ', 27,2)     
         call add_i_me  (n, tonoff, 'TRIP    ', '   STATUS   ', 27)        
         call add_i_me  (n, oldtrp, 'OTRP    ', 'OLDTRP      ', 27)     

!      
!-----------------------------------------------------------------------
!  Add two-region non equilibrium volume related minor edit flags
!-----------------------------------------------------------------------
!
         reg_test      = -num_neq  ! use nevlno to do region test
!
         call add_blk_entry (blk_num, nevlno, reg_test, 'NEQV    ')         
!         
         call add_r_me (n, nelm  , 'NELM    ', 'LIQ REG MASS',  8)        
         call add_r_me (n, nele  , 'NELE    ', 'LIQ REG ENER', 10)        
         call add_r_me (n, nelh  , 'NELH    ', 'LIQ REG ENTH',  7)        
         call add_r_me (n, nelvol, 'NLVO    ', 'LIQ REG VOL ', 16)        
         call add_r_me (n, nelt  , 'NELT    ', 'LIQ REG TEMP',  1)        
         call add_r_me (n, nelv  , 'NELV    ', 'LIQ REG SP V',  6)        
         call add_r_me (n, nelx  , 'NELX    ', 'LIQ REG QUAL', 27)        
         call add_r_me (n, nevv  , 'NEVV    ', 'VAP REG SP V',  6)        
         call add_r_me (n, nevm  , 'NEVM    ', 'VAP REG MASS',  8)        
         call add_r_me (n, neve  , 'NEVE    ', 'VAP REG ENER', 10)        
         call add_r_me (n, nevh  , 'NEVH    ', 'VAP REG ENTH',  7)        
         call add_r_me (n, nevt  , 'NEVT    ', 'VAP REG TEMP',  1)        
         call add_r_me (n, nevx  , 'NEVX    ', 'VAP REG QUAL', 27)        
         call add_i_me (n, nevlno, 'NVOL    ', 'NEVLNO      ', 27)   
         call add_r_me (n, qliqid, 'QLIQ    ', 'LIQ REG POWR', 13)        
         call add_r_me (n, qvapor, 'QVAP    ', 'VAP REG POWR', 13)
!
!-----------------------------------------------------------------------
!  Add temperature transport volume related minor edit flags
!--------------------------------------------------------------- --------
!
         reg_test      =  num_ttrn ! use itvold to do region test
!
         call add_blk_entry (blk_num, itvold, reg_test, 'TTVL    ')         
!
         call add_r_me (n, min1  , 'MIN*    ', 'MIN         ', 27)        
         call add_r_me (n, ein   , 'EIN*    ', 'EIN         ', 27)        
         call add_r_me (n, mout  , 'MOUT    ', 'MOUT        ', 27)        
         call add_r_me (n, eout  , 'EOUT    ', 'EOUT        ', 27)        
         call add_r_me (n, shfts , 'SHFT    ', 'SHFTS       ', 27)        
         call add_i_me (n, islug , 'ISLG    ', 'ISLUG       ', 27)     
         call add_i_me (n, mesh  , 'MESH    ', 'MESH        ', 27)     
         call add_i_me (n, itvold, 'ITVL    ', 'ITVOLD      ', 27)     
!
!-----------------------------------------------------------------------
!  Add control block related minor edit flags
!-----------------------------------------------------------------------
!
         reg_test      =  -(nci+ncb) ! use idcedt to do region test
!
         call add_blk_entry (blk_num, idcedt, reg_test, 'CBLK    ')         
!                  
         call add_r_me (n, cout  , 'COUT    ', '   OUTPUT   ', 27)        
!
!-----------------------------------------------------------------------
!  Add DNB related minor edit flags
!--------------------------------------------------------------- --------
         reg_test      =  0   ! no region test required use idummy as region
!
         call add_blk_entry (blk_num, idummy, reg_test, 'AUX DNB ')         
!         
         call add_r_me (n, hgrco , 'HGRC    ', 'AVG COR LHGR', 24)        
         call add_r_me (n, dnbrco, 'DNBC    ', 'AVG COR DNBR', 27)        
         call add_r_me (n, hgrmnz, 'HGRM    ', 'MIN. HT LHGR', 27)        
         call add_r_me (n, dnbrmz, 'DNBM    ', 'MIN. HT DNBR', 27)        
         call add_r_me (n, hgrhot, 'HGRH    ', 'HOT SPT LHGR', 24)        
         call add_r_me (n, dnbrht, 'DNBH    ', 'HOT SPT DNBR', 27)        
         call add_r_me (n, zmdnb , 'ZDNB    ', 'MIN DNBR POS', 15)        
         call add_r_me (n, hgrmd , 'HGRZ    ', 'MIN DNB LHGR', 24)        
         call add_r_me (n, dnbrm , 'DNBZ    ', '  MIN DNBR  ', 27)        
         call add_i_me (n, ndelt , 'NDLT    ', 'NDELT       ', 27)     
         call add_i_me (n, ndnb  , 'NDNB    ', 'NDNB        ', 27)     
!
!-----------------------------------------------------------------------
!  Add general transport volume related minor edit flags
!-----------------------------------------------------------------------
         reg_test      =  -num_gvol  ! use ivtrns to do region test
!
         call add_blk_entry (blk_num, ivtrns, reg_test, 'GTVL    ')         
!
         call add_r_me (n, convol, 'CONV    ', 'CONCENTRAT. ', 25)        
         call add_r_me (n, ttrnm , 'WEIT    ', 'MAT. WEIGHT ',  8)        
         call add_r_me (n, addmss, 'ADDM    ', '  SUM BIAS  ',  8)        
         call add_r_me (n, gmmssn, 'GMN     ', ' PREV MED MS',  8)        
         call add_i_me (n, ivtrns, 'IVTR    ', 'IVTR        ', 27)     
!             
      end subroutine add_me_list
!      
!================================================================================================================================
!
      end module minor_edit_search
