!$Header$
!$Copyright$
!
!  ==============================================================================================================================
!  Specifications and procedures used to create a list of allocated pointer blocks and related procedures for writing and reading
!  the blocks to and from a restart file.  The block list is also used when restoring to a given restart record.
!
!  This module also includes the following subroutines
!
!     write_restart_header
!     write_restart_record
!     write_restart_trailer
!     edit_restart_statistics
!     open_restart_file
!     read_static_blocks
!     read_restart_record
!
!  ==============================================================================================================================
!
      module restart
!
      use kind_specs
!
      integer(kind=ikd), save    :: &
       record_number=0,  &                   !  current restart record number
          rst_record=0,  &                   !  restart file record number where restart is to begin
              rst_in=13, &                   !  fortran unit used to read old restart file
             rst_out=14, &                   !  fortran unit used to write restart file - if = 0, no file will be written
             rst_eof=0,  &                   !  rst_in eof flag -- =1 if eof encountered -- otherwise =0
            rst_tlbl=0                       !  rst_in trailer label flag -- =1 if trailer label encountered -- otherwise =0
!                
      real(kind=rkd),    pointer :: r_rw(:)  ! pointer array used to write or read real restart blocks
      integer(kind=ikd), pointer :: i_rw(:)  ! pointer array used to write or read integer restart blocks
      logical(kind=lkd), pointer :: l_rw(:)  ! pointer array used to write or read logical restart blocks
!
!  The following variables contain the restart file label information for the file last opened 
!
      integer(kind=ikd), save ::  &
              i_len, &   ! maximum length for integer restart blocks
             i2_len, &   ! maximum length for 2D integer restart blocks
              l_len, &   ! maximum length for logical restart blocks
             l2_len, &   ! maximum length for 2D logical restart blocks
              r_len, &   ! maximum length for real restart blocks
             r2_len      ! maximum length for 2D real restart blocks
!              
      character(kind=ckd,len=32), save ::    &
          hdr_label, &   !  file header label on restart file
          lic_org        !  name of licensed organization on restart file
!          
      character(kind=ckd,len=80), save ::    &
         prob_title, &   !  problem title read from restart tape
         ver_header      !  code version header read from restart tape
!
!  The following information is used save problem dimension information for restart run
      integer(kind=ikd) :: iup
      parameter (iup    = 6 )
      integer(kind=ikd), save :: isav(iup)  
      contains
!
!================================================================================================================================
      subroutine write_restart_header
!
!  Subroutine to write the header record to the restart file on unit iunit.  It also writes the static restart blocks to the 
!  restart file following the header record.
!
!  write_restart_header is called from edit and inrstr
!
      use kind_specs
      use inp_cards
      use edit_headers
      use restart_block_list
!
      implicit none
!
!  Local Variables
!      
      character(kind=ckd,len=6)     file_name
      character(kind=ckd,len=8)     blk_name                    !  name of block
      real(kind=rkd),    pointer :: r_pntr(:), r2_pntr(:,:)     !  dummy pointers
      integer(kind=ikd), pointer :: i_pntr(:), i2_pntr(:,:)     !  dummy pointers
      logical(kind=lkd), pointer :: l_pntr(:), l2_pntr(:,:)     !  dummy pointers
      integer(kind=ikd)  l, len_blk, l2, itype, jtype, n, stat
      logical(kind=lkd) opn
!
!  Return if new restart file is not wanted
!
      if ( rst_out==0 ) return      
!
!  Return if already file is already open      
!
      inquire (rst_out,opened=opn, err=120, iostat=stat)
!
      if ( opn ) then
         return
      end if
!
!  Open restart file      
!
      write (file_name,"('TAPE',i2)") rst_out
!      
      open (rst_out, file=file_name, form='unformatted', status='unknown', err=110, iostat=stat)
!
!  Write restart file header
!
      write (rst_out,iostat=stat,err=100) header_label, licensed_org, version_header, problem_title,  &
             mxi_len, mxi2_len, mxl_len, mxl2_len, mxr_len, mxr2_len, blk
!
!  Write static blocks for temporary use. 
!  This information will be used during restart run. When ret_input is called space for the restart block list 
!  will be allocated. inp_card array has different space for original and restart run, hence during restart run
!  space for inp cards will be allocated and restart block list is created,  then inp card arrays are read and
!  information will be transferred to restart cards. Since restart block lists are already created, after opening the
!  restart file, header info will be read and the temporary static blocks are read. This gives original static block info.
!  if inp cards restart block is encountered. the inp cards will be deallocated and reallocated with additional space. 
!     
!  Write static header blocks - the original problem dimensions and original INP free form input table array 
!  are included in the static blocks
!
      do n = 1, blk
         jtype = rest_list(n) % block_type
         if ( jtype/=1 ) cycle     !  skip all but static blocks to be written to header
!
         blk_name = rest_list(n) % block_name
         len_blk  = rest_list(n) % block_size
!
         write (rst_out,iostat=stat,err=100) blk_name, len_blk
!
         itype    = rest_list(n) % pntr_type
!
!     Write pointer array
!         
         select case (itype)
         case(1)
            r_pntr => rest_list(n) % r_restrt_pntr
!            
            write (rst_out,iostat=stat,err=100) (r_pntr(l), l=1,len_blk)
         case(2)
            i_pntr  => rest_list(n) % i_restrt_pntr
!            
            write (rst_out,iostat=stat,err=100) (i_pntr(l), l=1,len_blk)
         case(3)
            l_pntr  => rest_list(n) % l_restrt_pntr
!            
            write (rst_out,iostat=stat,err=100) (l_pntr(l), l=1,len_blk)
         case(4)
            r2_pntr  => rest_list(n) % r2_restrt_pntr
!            
            write (rst_out,iostat=stat,err=100) ((r2_pntr(l,l2), l=1,len_blk), l2=1,2)
         case(5)
            i2_pntr => rest_list(n) % i2_restrt_pntr
!            
            write (rst_out,iostat=stat,err=100) ((i2_pntr(l,l2), l=1,len_blk), l2=1,2)
         case(6)
            l2_pntr => rest_list(n) % l2_restrt_pntr
!            
            write (rst_out,iostat=stat,err=100) ((l2_pntr(l,l2), l=1,len_blk), l2=1,2)
         end select
!
      end do
!
!  Initialize restart record number
!
      record_number = 0
      return
!
!  Write error occured - edit statistics
!                
  100 write (6,500) rst_out, stat
      call fabend
!
!  Open error occured on restart file - edit statistics
!                
  110 write (6,510) rst_out, blk_name, len_blk, stat
      call fabend
!
!  Error checking open status for restart file in subroutine write_restart_header - edit statistics
!                
  120 write (6,520) rst_out, opn, stat
      call fabend
!  
  500 format (/' ******** Error writing header record to restart file writing.  Details follow.'/  &
               '          Fortran unit  = ',i8/  &
               '          I/O status    = ',i8/)
  510 format (/' ******** Error opening restart file for writing.  Details follow.'/  &
               '          Fortran unit  = ',i8/  &
               '          Block name    = ',a8/  &
               '          Block length  = ',i8/  &
               '          I/O status    = ',i8/)
  520 format (/' ******** Error checking open status in subroutine write_restart_header.  Details follow.'/  &
               '          Fortran unit  = ',i8/  &
               '          Open status   = ',l8/  &
               '          I/O status    = ',i8/)
!               
      end subroutine write_restart_header
!
!================================================================================================================================
      subroutine write_restart_record
!
!  Subroutine to write a restart record to the restart file.
!
!  write_restart_record is called by subroutine edit
!
      use kind_specs
      use inp_cards
      use edit_headers
      use restart_block_list
!
      implicit none
!
!  Local variables
!
      character(kind=ckd,len=8)     blk_name                    !  name of block
      real(kind=rkd),    pointer :: r_pntr(:), r2_pntr(:,:)     !  dummy pointers
      integer(kind=ikd), pointer :: i_pntr(:), i2_pntr(:,:)     !  dummy pointers
      logical(kind=lkd), pointer :: l_pntr(:), l2_pntr(:,:)     !  dummy pointers
      integer(kind=ikd)  l, len_blk, l2, itype, jtype, n, status
!
!  Write restart record header and number
!
      record_number = record_number + 1
      blk_name      = data_record
      len_blk       = record_number
!
      if ( rst_out==0 ) return      
!      
      write (rst_out,iostat=status,err=100) blk_name, len_blk
!
!  Write dynamic blocks to restart record
!
      do n = 1, blk
         jtype = rest_list(n) % block_type
         if ( jtype/=2 ) cycle     !  skip all but dynamic blocks to be written as part of restart record
!
         blk_name = rest_list(n) % block_name
         len_blk  = rest_list(n) % block_size
!
         write (rst_out,iostat=status,err=100) blk_name, len_blk
!
         itype    = rest_list(n) % pntr_type
!
!     Write pointer array
!         
         select case (itype)
         case(1)
            r_pntr => rest_list(n) % r_restrt_pntr
!            
            write (rst_out,iostat=status,err=100) (r_pntr(l), l=1,len_blk)
         case(2)
            i_pntr => rest_list(n) % i_restrt_pntr
!            
            write (rst_out,iostat=status,err=100) (i_pntr(l), l=1,len_blk)
         case(3)
            l_pntr => rest_list(n) % l_restrt_pntr
!            
            write (rst_out,iostat=status,err=100) (l_pntr(l), l=1,len_blk)
         case(4)
            r2_pntr  => rest_list(n) % r2_restrt_pntr
!            
            write (rst_out,iostat=status,err=100) ((r2_pntr(l,l2), l=1,len_blk), l2=1,2)
         case(5)
            i2_pntr => rest_list(n) % i2_restrt_pntr
!            
            write (rst_out,iostat=status,err=100) ((i2_pntr(l,l2), l=1,len_blk), l2=1,2)
         case(6)
            l2_pntr => rest_list(n) % l2_restrt_pntr
!            
            write (rst_out,iostat=status,err=100) ((l2_pntr(l,l2), l=1,len_blk), l2=1,2)
         end select
!
      end do
!
      return
!
!  Write error occured - edit statistics
!                
  100 write (6,500) rst_out, record_number, blk_name, len_blk, status
      call fabend
!  
  500 format (/' ******** Error writting restart record for restart file.  Details follow.'/  &
               '          Fortran unit =  ',i8/  &
               '          Record number = ',i8/  &
               '          Block name   =  ',a8/  &
               '          Block length =  ',i8/  &
               '          I/O status   =  ',i8/)
!               
      end subroutine write_restart_record
!
!================================================================================================================================
      subroutine write_restart_trailer
!
!  Subroutine to write a restart trailer record to the restart file.
!
!  write_restart_trailer is called from subroutine edit and retran (entry point restrt)
!
      use kind_specs
      use inp_cards
      use edit_headers
!
      implicit none
!
!  Local variables
!
      integer(kind=ikd)  len_blk, status
      character(kind=ckd,len=8)     blk_name      !  name of block
!
!  Write restart trailer record header
!
      blk_name = 'trailer '
      len_blk  = 1
!
      if ( rst_out==0 ) return      
!
      write (rst_out,iostat=status,err=100) blk_name, len_blk
!
      write (rst_out,iostat=status,err=100) trailer_label
      endfile (rst_out,iostat=status,err=100)
      close (rst_out)
!
      return      
!
!  Write error occured - edit statistics
!                
  100 write (6,500) rst_out, blk_name, len_blk, status
      call fabend
!  
  500 format (/' ******** Error writting restart trailer record for restart file.  Details follow.'/  &
               '          Fortran unit =  ',i8/  &
               '          Block name   =  ',a8/  &
               '          Block length =  ',i8/  &
               '          I/O status   =  ',i8/)
!               
      end subroutine write_restart_trailer
!
!================================================================================================================================
      subroutine edit_restart_statistics
!
!  Subroutine to edit the statistics for a restart file.
!
!  edit_restart_statistics is called from subroutine retran and its entry point restrt
!
      use kind_specs
      use edit_headers
      use restart_block_list      
!
      implicit none
!
!  Local variables
!
      integer(kind=ikd)  k, loop, m, n, od
      character(kind=ckd,len=8) blk_name
!
      if ( rst_out==0 ) return      
!
!  Write restart file header information
!
      write (6,500) header_label, licensed_org, version_header, problem_title, mxi_len, mxi2_len, mxl_len, mxl2_len,  &
                    mxr_len, mxr2_len
!
!  Write block statistics
!
      loop   = blk/2
      od     = blk - 2*loop
      k      = loop + od
!      
      do n = 1, loop
         m      = n + k
!
         write (6,510) n, rest_list(n) % block_type, rest_list(n) % block_name,  &
                          rest_list(n) % block_size, rest_list(n) % pntr_type ,  &
                       m, rest_list(m) % block_type, rest_list(m) % block_name,  &
                          rest_list(m) % block_size, rest_list(m) % pntr_type    
!
      end do
!
!  Write odd line if necessary
!  
      if ( od>0 ) then
         write (6,510) k, rest_list(k) % block_type, rest_list(k) % block_name,  &
                          rest_list(k) % block_size, rest_list(k) % pntr_type 
      end if
!
      write (6,520)      
!      
  500 format (//' RETRAN-3D RESTART FILE STATISTICS '/  &
                ' ========================================================================================================'/  &
                ' HEADER LABEL             : ',A32/ &
                ' LICENSED ORGANIZATION    : ',A32/ &
                ' CODE VERSION             : ',A80/ &
                ' PROBLEM TITLE            : ',A80/ &
                ' MAX INTGER BLOCK LEN     : ',I8/  &
                ' MAX INTGER 2D BLOCK LEN  : ',I8/  &
                ' MAX LOGICAL BLOCK LEN    : ',I8/  &
                ' MAX LOGICAL 2D BLOCK LEN : ',I8/  &
                ' MAX REAL BLOCK LEN       : ',I8/  &
                ' MAX REAL 2D BLOCK LEN    : ',I8// &
           5x,2(' -----------   BLOCK   ----------   POINTER ',10x)/ &
           5x,2(' NUM    TYPE   NAME        LENGTH   TYPE    ',10x))
  510 format (5x,2(I4,5X,I2,4X,A8,3X,I7,4X,I2,15x))
  520 format (//)
!               
      end subroutine edit_restart_statistics
!
!================================================================================================================================
      subroutine open_restart_file (iunit)
!
!  Subroutine to open a restart file on unit iunit.
!
!  open_restart_file is called from inrstr and tapebc
!
!     iunit  = fortran unit for restart file to open for reading
!
      use kind_specs
      use inp_cards
      use edit_headers
      use restart_block_list
!
      implicit none
! 
!  Dummy arguments
!
      integer(kind=ikd)  iunit
      intent(in)         iunit
!
!  Local variables
! 
      character(kind=ckd,len=6) file_name      
!         
      character(kind=ckd,len=8)     blk_name      !  name of block
      integer(kind=ikd)  l, l2, len_blk, stat, n, status, itype, jtype, block
      real(kind=rkd),    pointer :: r_pntr(:), r2_pntr(:,:)     !  dummy pointers
      integer(kind=ikd), pointer :: i_pntr(:), i2_pntr(:,:)     !  dummy pointers
      logical(kind=lkd), pointer :: l_pntr(:), l2_pntr(:,:)     !  dummy pointers      
!
!  Open restart file on unit iunit with file name TAPE[iunit]     
!
      write (file_name,'(a4,i2)') 'TAPE', iunit
!      
      open (iunit, file=file_name, form='unformatted', status='old', err=100, iostat=stat)
!
!  Read restart file header
!
      read (iunit,iostat=stat,err=100,end=110) hdr_label, lic_org, ver_header, prob_title, i_len, i2_len,   &
            l_len, l2_len, r_len, r2_len, r_blk
!
!  Insure that file is a restart file
!
      if ( hdr_label/=header_label ) then
         write (6,520) iunit, header_label, hdr_label, lic_org, ver_header, prob_title, run_date
         call fabend
      end if
!
!  Check that first 16 characters of title match current problem title for restart file (iunit=13)
!  
      if ( iunit==14 .and. prob_title(1:16)/=problem_title(1:16) ) then
         write (6,530) iunit, hdr_label, prob_title, problem_title, run_date
         call fabend
      end if
!    
      return
!      
!  Read error occured - edit statistics
!                
  100 write (6,500)  file_name, iunit, stat
      call fabend
!
!  End of file encountered when reading header record - edit statistics
!                
  110 write (6,510) iunit, blk_name, len_blk, stat
      call fabend
!
!  Read error occured - edit statistics
!                
 1000 write (6,600) rst_in, record_number, blk_name, len_blk, status
      call fabend
!
!  End of file encountered when reading restart record - edit statistics
!                
 1100 write (6,610) rst_in, record_number, blk_name, len_blk, status
      rst_eof = 1
!      
!  Trailer label encountered while reading data - write error message and terminate
!      
 1200 continue
      write (6,630) rst_in, record_number
      rst_tlbl = 1
!      
      return
!  
  500 format (/' ******** Error opening restart file for reading.  Details follow.'/  &
               '          File name     = ',a6/  &
               '          Fortran unit  = ',i8/  &
               '          I/O status    = ',i8//)
  510 format (/' ******** Error - end of file encountered when reading restart file header record.  Details follow.'/  &
               '          Fortran unit  = ',i8/  &
               '          Block name    = ',a8/  &
               '          Block length  = ',i8/  &
               '          I/O status    = ',i8//)
  520 format (/' ******** Error - header record read indicates file is not a restart file. Details follow.'/  &
               '          Fortran unit         = ', i8/ &
               '          Retran header label  = ',a32/ &
               '          Information read from the restart file follows'  &
               '          =============================================='  &
               '          Header label         = ',a32/ &
               '          Licensed orginzation = ',a32/ &
               '          Code version used    = ',a80/ &
               '          Problem title        = ',a80/ &
               '          Run date             = ',a8//)
  530 format (/' ******** Error - first 16 characters of problem title on restart file do not match the current title.  '  &
               '          Details follow.'/  &
               '          Fortran unit          = ', i8/  &
               '          Restart header label  = ',a32/ &
               '          Current problem title = ',a80/ &
               '          Information read from the restart file follows'  &
               '          =============================================='  &
               '          Problem title         = ',a80/ &
               '          Run date              = ',a8//)
!               
  600 format (/' ******** Error reading restart record for restart file.  Details follow.'/  &
               '          Fortran unit =  ',i8/  &
               '          Record number = ',i8/  &
               '          Block name   =  ',a8/  &
               '          Block length =  ',i8/  &
               '          I/O status   =  ',i8/)
  610 format (/' ******** Error - end of file encountered when reading restart record.  Details follow.'/  &
               '          Fortran unit  = ',i8/  &
               '          Record number = ',i8/  &
               '          Block name    = ',a8/  &
               '          Block length  = ',i8/  &
               '          I/O status    = ',i8//)
  620 format (/' ******** Error - block miss match when reading restart file.  This indicates that the restart ', &
               'file is not'/  &
               '          compatible with the code version being used. Details follow.'/  &
               '          Fortran unit = ',i8/  &  
               '          Expected block name   = ',a8/  &
               '          Block name read       = ',a8/  &
               '          Block length expected = ',i8/  &
               '          Block length read     = ',i8//)
  630 format (/' ******** Error - trailer label encountered when reading restart record.  Details follow.'/  &
               '          Fortran unit              = ',i8/  &
               '          Looking for record number = ',i8//)
!             
      end subroutine open_restart_file
!
!================================================================================================================================
      subroutine read_static_blocks
!
!  Subroutine to read the header and static data blocks from the restart file.  
!
!  read_static_blocks is called from inrstr
!
      use kind_specs
      use inp_cards
      use edit_headers
      use restart_block_list
      use retran_configuration
!
      implicit none
!
!  Local variables
!
      character(kind=ckd,len=8)     blk_name, b_name            !  name of block
      real(kind=rkd),    pointer :: r_pntr(:), r2_pntr(:,:)     !  dummy pointers
      integer(kind=ikd), pointer :: i_pntr(:), i2_pntr(:,:)     !  dummy pointers
      logical(kind=lkd), pointer :: l_pntr(:), l2_pntr(:,:)     !  dummy pointers
      integer(kind=ikd)  l, len_blk, l2, itype, jtype, n, stat, status
!
!  Read static header blocks - the original problem dimensions and original INP free form input table array 
!  are included in the static blocks
!
      b_name = 'r_cards '
!      
      do n = 1, r_blk
         jtype = rest_list(n) % block_type
         if ( jtype/=1 ) cycle     !  include only static blocks in the restart file header
!
         blk_name = rest_list(n) % block_name
         len_blk  = rest_list(n) % block_size
!
         read (rst_in,iostat=stat,err=100,end=110) blk_name, len_blk
!
         if ( blk_name==trailer_label ) then
            go to 120
         end if
!         
!  If INP free form input data block, see if space needs to be allocated to read block into
!
         if ( rest_list(n)%block_name .eq. b_name ) then
!         
!  Deallocate space for original INP table and data array (inp_tbl & rdata)
             deallocate (r_cards, stat=status)
!
!         Check status of de-allocate - write error if de-allocate failed
             if (status .ne. 0) then
                blk_name   = 'real'
                write (6,505) blk_name, len_tbl, status
                call fail
             endif     
             len_tbl = 0         
!         allow for some additional space for replacement card - generalized restart             
             len_tbl = len_blk + len_rblk   
             call allok8_inp_card_arrays               
!          
         end if
!         
!     Read pointer array
!         
         itype    = rest_list(n) % pntr_type
         select case (itype)
         case(1)
             r_pntr => rest_list(n) % r_restrt_pntr
!            
            read (rst_in,iostat=stat,err=100,end=110) (r_pntr(l), l=1,len_blk)
         case(2)
            i_pntr => rest_list(n) % i_restrt_pntr
!            
            read (rst_in,iostat=stat,err=100,end=110) (i_pntr(l), l=1,len_blk)
         case(3)
            l_pntr => rest_list(n) % l_restrt_pntr
!            
            read (rst_in,iostat=stat,err=100,end=110) (l_pntr(l), l=1,len_blk)
         case(4)
            r2_pntr => rest_list(n) % r2_restrt_pntr
!            
            read (rst_in,iostat=stat,err=100,end=110) ((r2_pntr(l,l2), l=1,len_blk), l2=1,2)
         case(5)
            i2_pntr => rest_list(n) % i2_restrt_pntr
!            
            read (rst_in,iostat=stat,err=100,end=110) ((i2_pntr(l,l2), l=1,len_blk), l2=1,2)
         case(6)
            l2_pntr => rest_list(n) % l2_restrt_pntr
!            
            read (rst_in,iostat=stat,err=100,end=110) ((l2_pntr(l,l2), l=1,len_blk), l2=1,2)
         end select
!
      end do
      return
!
!  Read error occured - edit statistics
!                
  100 write (6,500) rst_in, stat
      call fabend
!
!  Open error occured on restart file - edit statistics
!                
  110 write (6,510) rst_in, blk_name, len_blk, stat
      call fabend
!
!  Trailer label encountered while reading data - write error message and terminate
!      
  120 continue
      write (6,520) rst_in
      call fabend
!            
  500 format (/' ******** Error reading restart file header or static blocks.  Details follow.'/  &
               '          Fortran unit  = ',i8/  &
               '          I/O status    = ',i8/)
!
  505 format (/' ******** Error de-allocating space for ',a8,' INP free form processing array for restart input data.'/ &
              10x,'Block length = ',i8/ &
              10x,'Status flag returned from allocate = ',i3)               
  510 format (/' ******** Error reading restart file.  Details follow.'/  &
               '          Fortran unit  = ',i8/  &
               '          Block name    = ',a8/  &
               '          Block length  = ',i8/  &
               '          I/O status    = ',i8/)
  520 format (/' ******** Error - trailer label encountered when reading restart file while attempting to ', &
               ' find the '/ &
               '          original problem input data.'/  &
               '          Fortran unit              = ',i8//)
!               
      end subroutine read_static_blocks
!
!================================================================================================================================
      subroutine read_restart_record (get_rec)
!
!  Subroutine to read a specified restart record from the restart file.
!
!  read_restart_record is called by retran (entry point restrt)
!
!     get_rec = restart record to read from restart file
!
      use kind_specs
      use edit_headers
      use restart_block_list
!
      implicit none
!
!  Dummy arguments
!
      integer(kind=ikd) get_rec
      intent (in)       get_rec
!
!  Local variables
! 
      character(kind=ckd,len=8)     blk_name                     !  name of block
      real(kind=rkd),    pointer :: r_pntr(:), r2_pntr(:,:)      !  dummy pointers
      integer(kind=ikd), pointer :: i_pntr(:), i2_pntr(:,:)      !  dummy pointers
      logical(kind=lkd), pointer :: l_pntr(:), l2_pntr(:,:)      !  dummy pointers
      integer(kind=ikd)  l, len_blk, l2, itype, jtype, n, rec, status
!
!  Read restart record header and number - skip to record get_rec
!
      rec      = 0
      blk_name = '        '
!
      do while (rec<get_rec)               !  terminate loop when rec = get_rec
         do while (blk_name/=data_record)  !  terminate loop when blk_name = data_record   
!
            read (rst_in,iostat=status,err=100,end=110) blk_name, rec  
!           
            if ( blk_name==trailer_label ) then
               go to 120
!
!        Skip the associated data block
!
            else if ( blk_name/=data_record ) then
               read (rst_in,iostat=status,err=100,end=110) 
            end if           
!            
         end do
      end do
!
!  Read dynamic blocks from restart record.  Blocks (dynamic) read from the restart file must match those in the 
!  restart block list, otherwise there is an error.
!
      do n = 1, blk
         jtype = rest_list(n) % block_type
         if ( jtype/=2 ) cycle     !  skip all but dynamic blocks to be written as part of restart record
!
         read (rst_in,iostat=status,err=100,end=110) blk_name, len_blk
!
         if ( blk_name/=rest_list(n)%block_name .or. len_blk/=rest_list(n)%block_size ) then
            write (6,520) rst_in, rest_list(n)%block_name, blk_name, rest_list(n)%block_size, len_blk
            call fabend
         else if ( blk_name==trailer_label ) then
            go to 120
         end if
!         
!     Read into pointer array
!         
         itype    = rest_list(n) % pntr_type
!
         select case (itype)
         case(1)
            r_pntr  => rest_list(n) % r_restrt_pntr
!            
            read (rst_in,iostat=status,err=100,end=110) (r_pntr(l), l=1,len_blk)
         case(2)
            i_pntr  => rest_list(n) % i_restrt_pntr
!            
            read (rst_in,iostat=status,err=100,end=110) (i_pntr(l), l=1,len_blk)
         case(3)
            l_pntr  => rest_list(n) % l_restrt_pntr
!            
            read (rst_in,iostat=status,err=100,end=110) (l_pntr(l), l=1,len_blk)
         case(4)
            r2_pntr => rest_list(n) % r2_restrt_pntr
!            
            read (rst_in,iostat=status,err=100,end=110) ((r2_pntr(l,l2), l=1,len_blk), l2=1,2)
         case(5)
            i2_pntr => rest_list(n) % i2_restrt_pntr
!            
            read (rst_in,iostat=status,err=100,end=110) ((i2_pntr(l,l2), l=1,len_blk), l2=1,2)
         case(6)
            l2_pntr => rest_list(n) % l2_restrt_pntr
!            
            read (rst_in,iostat=status,err=100,end=110) ((l2_pntr(l,l2), l=1,len_blk), l2=1,2)
         end select
!
      end do
!
      return
!
!  Read error occured - edit statistics
!                
  100 write (6,500) rst_in, record_number, blk_name, len_blk, status
      call fabend
!
!  End of file encountered when reading restart record - edit statistics
!                
  110 write (6,510) rst_in, get_rec, blk_name, len_blk, status
      rst_eof = 1
!
!  Trailer label encountered while reading data - write error message and terminate
!      
  120 continue
      write (6,530) rst_in, get_rec
      rst_tlbl = 1
!  
  500 format (/' ******** Error reading restart record for restart file.  Details follow.'/  &
               '          Fortran unit =  ',i8/  &
               '          Record number = ',i8/  &
               '          Block name   =  ',a8/  &
               '          Block length =  ',i8/  &
               '          I/O status   =  ',i8/)
  510 format (/' ******** Error - end of file encountered when reading restart record.  Details follow.'/  &
               '          Fortran unit  = ',i8/  &
               '          Record number = ',i8/  &
               '          Block name    = ',a8/  &
               '          Block length  = ',i8/  &
               '          I/O status    = ',i8//)
  520 format (/' ******** Error - block miss match when reading restart file.  This indicates that the restart ', &
               'file is not'/  &
               '          compatible with the code version being used. Details follow.'/  &
               '          Restart record        = ',i8/  &  
               '          Expected block name   = ',a8/  &
               '          Block name read       = ',a8/  &
               '          Block length expected = ',i8/  &
               '          Block length read     = ',i8//)
  530 format (/' ******** Error - trailer label encountered when reading restart record.  Details follow.'/  &
               '          Fortran unit              = ',i8/  &
               '          Looking for record number = ',i8//) 
!               
      end subroutine read_restart_record
!
      end module restart


