!$Header$
!$Copyright$
!
!  ==============================================================================================================================
!  Specifications and procedures used to create a list of allocated pointer blocks and related procedures for writing 
!  the blocks to a restart file.  The block list is also used when restoring to a given restart record.
!
!  Module restart_block_list also contains the following subroutine(s)
!
!     build_restart_block_list
!     build_restart_block_list_2d (entry point)
!
!  ==============================================================================================================================
!
      module restart_block_list
!
      use kind_specs
      use retran_configuration
!
      integer(kind=ikd), save    :: &
                 blk=0,  &    !  number of blocks in restart block list
               r_blk=0,  &    !  number of blocks in restart block list, used during restart
             mxi_len=0,  &    !  maximum length of any integer block
            mxi2_len=0,  &    !  maximum length of any 2D integer block
             mxl_len=0,  &    !  maximum length of any logical block
            mxl2_len=0,  &    !  maximum length of any 2D logical block
             mxr_len=0,  &    !  maximum length of any real block
            mxr2_len=0,  &    !  maximum length of any 2D real block
          num_blocks=0        !  size of restart block list - used to allocate space
!          
      logical(kind=lkd), save    :: &
        save_restart=.false.  !  logical flag indicating if a restart file will be saved (.true.)
!
!  Derived type specification used to define restart block list
!
      type block_list
         character(kind=ckd,len=12) :: block_name            ! name of restart block - used to locate specific blocks for tdvs
         real(kind=rkd),    pointer :: r_restrt_pntr(:),  &  ! pointer for real restart block
                                       r2_restrt_pntr(:,:)   ! pointer for real 2D restart block
         integer(kind=ikd), pointer :: i_restrt_pntr(:),  &  ! pointer for integer restart block
                                       i2_restrt_pntr(:,:)   ! pointer for integer 2D restart block
         logical(kind=lkd), pointer :: l_restrt_pntr(:),  &  ! pointer for logical restart block
                                       l2_restrt_pntr(:,:)   ! pointer for logical 2D restart block
         integer(kind=ikd)          :: block_size,        &  ! length of restart block
                                       pntr_type,         &  ! 1D pointer type flag -- 1=real / 2=integer / 3=logical
                                                             ! 2D pointer type flag -- 4=real / 5=integer / 6=logical
                                       block_type            ! block type flag      -- 1=static / 2=dynamic
      end type block_list
!
      type (block_list), target, allocatable, save :: rest_list(:)
!
      contains
!
!================================================================================================================================
      subroutine build_restart_block_list (blk_name, r_pntr, i_pntr, l_pntr, len_blk, itype, jtype)
!
!  Subroutine to build a list of pointers and lengths for allocated block arrays that are to be written to 
!  the restart file.  Subroutine build_restart_list must be called after allocating any data block that 
!  needs to be included in the restart file.
!
!     Note: Prior to the first call, num_blocks and save_restart must be defined.
!
!     blk_name  = name of block                                                                    input
!     r_pntr    = pointer to real block                                                            input
!     i_pntr    = pointer to integer block                                                         input
!     l_pntr    = pointer to logical block                                                         input
!     len_blk   = length of the block                                                              input
!     itype     = flag indicating type specificaion of the block pointer to use                    input
!                 1 = real    (integer and logical pointers not used - dummies supplied)
!                 2 = integer (real and logical pointers not used - dummies supplied)
!                 3 = logical (real and integer pointers not used - dummies supplied)
!     jtype     = flag indicating type of data stored in block                                     input
!                 0 = static  -- will not be written to restart file
!                 1 = static  -- will be written to the header record of a restart file
!                 2 = dynamic -- will be written to each restart record
!
      use kind_specs
!
      integer(kind=ikd)  status                               !  status of allocate
      character(kind=ckd,len=8)    blk_name                   !  name of block
      real(kind=rkd),    pointer :: r_pntr(:), r2_pntr(:,:)   !  dummy pointers
      integer(kind=ikd), pointer :: i_pntr(:), i2_pntr(:,:)   !  dummy pointers
      logical(kind=lkd), pointer :: l_pntr(:), l2_pntr(:,:)   !  dummy pointers
      integer(kind=ikd)  blk_save, i, len_blk, itype, jtype, ktype
!
      ktype  = itype
      go to 5      
!
!==========================================================================================================
!      
      entry build_restart_block_list_2d (blk_name, r2_pntr, i2_pntr, l2_pntr, len_blk, itype, jtype)
!
!  Entry point to build a list of pointers and lengths for allocated block arrays that are to be written  
!  to the restart file.  Entry build_restart_list_2d makes list entries for 2D arrays and must be called 
!  after allocating any data block that needs to be included in the restart file.
!
!     Note: Prior to the first call, num_blocks and save_restart must be defined.
!
!     blk_name  = name of block                                                                   input
!     r2_pntr   = pointer to real 2D block                                                        input
!     i2_pntr   = pointer to integer 2D block                                                     input
!     l2_pntr   = pointer to logical 2D block                                                     input
!     len_blk   = length of the block                                                             input
!     itype     = flag indicating type specificaion of the block pointer to use                   input
!                 1 = real    (integer and logical pointers not used - dummies supplied)
!                 2 = integer (real and logical pointers not used - dummies supplied)
!                 3 = logical (real and integer pointers not used - dummies supplied)
!     jtype     = flag indicating type of data stored in block                                     input
!                 0 = static  -- will not be written to restart file
!                 1 = static  -- will be written to the header record of a restart file
!                 2 = dynamic -- will be written to each restart record
!
      ktype  = itype + 3
!
!==========================================================================================================
!      
!  Return if no restart file is to be saved
!
    5 continue
      num_blocks  = rstrt_blks
      if (.not. save_restart) then
         return
      endif
!
!  Allocate space for restart block list
!
      if (.not. allocated (rest_list)) then
         blk    = 0
!
         if (num_blocks .gt. 0) then
!
            allocate (rest_list(num_blocks), stat=status)
!
!        Check status of allocate
!
            if (status .ne. 0) then
               write (6,100) num_blocks, status
               call fail
            endif
!
!     num_blocks not initialized properly - can't allocate necessary space 
!
         else   
            status  = 0
            write (6,100) num_blocks, status
            call fail
            return
         endif
      endif
!
!  See if redefining an existing block
! 
      blk_save = blk + 1
!
      do i = 1, blk
         if (rest_list(i) % block_name .eq.  blk_name) then
            blk_save = blk
            blk      = i - 1
            go to 10
         endif
      enddo
!
   10 continue
      blk    = blk + 1
!
!  Make list entries
!
      if (blk .le. num_blocks) then
         select case (ktype)
         case(1)
            rest_list(blk) % r_restrt_pntr  => r_pntr
            mxr_len  = max (mxr_len, len_blk)
         case(2)
            rest_list(blk) % i_restrt_pntr  => i_pntr
            mxi_len  = max (mxi_len, len_blk)
         case(3)
            rest_list(blk) % l_restrt_pntr  => l_pntr
            mxl_len  = max (mxl_len, len_blk)
         case(4)
            rest_list(blk) % r2_restrt_pntr => r2_pntr
            mxr2_len = max (mxr2_len, len_blk)
         case(5)
            rest_list(blk) % i2_restrt_pntr => i2_pntr
            mxi2_len = max (mxi2_len, len_blk)
         case(6)
            rest_list(blk) % l2_restrt_pntr => l2_pntr
            mxl2_len = max (mxl2_len, len_blk)
         end select
!
         rest_list(blk) % block_name = blk_name
         rest_list(blk) % block_size = len_blk
         rest_list(blk) % pntr_type  = ktype
         rest_list(blk) % block_type = jtype
         blk                         = blk_save
!
!    Out of space for block list - write error and terminate
!
      else
         write (6, 110) num_blocks
         call fail
         return
      endif
!
  100 format (/'Error allocating space for list of restart data blocks (build_restart_block_list).'/   &
               'Length requested = ',i8/  &
               'Status flag returned from allocate = ',i3)
  110 format (/'Error building list of restart data blocks (build_restart_block_list).'/   &
               'Length allocated = ',i8, 'is not adequate - programming error.')
      end subroutine build_restart_block_list
!
      end module restart_block_list


