!$Header$
!$Copyright$
!
!  ===============================================================================================================================
!  Specifications and space allocations for turbine related arrays
!
!  Module restart_block_list also contains the following subroutine(s)
!
!     allok8_turb_shafts
!     allok8_turbines
!     allok8_turb_stages
!
!  ===============================================================================================================================
!
      module turbines
!
      use kind_specs 
      use restart_block_list 
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Specifications for turbine shaft related arrays - a shaft can have multiple turbines attached
! 
      integer(kind=ikd), save :: & 
          num_shft,  &  !  number of turbine shafts
         siz_rshft,  &  !  size of the array allocated for the real shft pointer arrays
         siz_ishft      !  size of the array allocated for the integer shft pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_shft(:)
!
!  All of the following real and integer pointer arrays are Rank (1:num_shft)
!  Space is allocated and pointers associations are made in subroutine allok8_shft
!
      real(kind=rkd), pointer, save :: &
       oshrpm(:), &  !  old time shaft speed.                             
       powout(:), &  !  power produced (actual computed power out)        
         shir(:), &  !  shaft moment of inertia.                          
        shpow(:), &  !  shaft demand power.                               
        shrpm(:), &  !  shaft speed (rpm)                                 
        shtor(:), &  !  shaft torque.                                     
       shtori(:)     !  initial shaft torque.                             
!       
      integer(kind=ikd), target, allocatable, save :: i_shft(:)
      integer(kind=ikd), pointer, save :: &
         isin(:), &  !  speed integration control block number            
        ntsha(:), &  !  turbine shaft number.                             
        nturs(:)     !  number of turbines on this shaft.                 
!
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for individual turbine description related arrays
!
      integer(kind=ikd), save :: & 
         num_turb,  &  !  number of turbines
         siz_rturb,  &  !  size of the array allocated for the real turb pointer arrays
         siz_iturb      !  size of the array allocated for the integer turb pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_turb(:)
!
!  All of the following real, integer and logical pointer arrays are Rank (1:num_turb)
!  Space is allocated and pointers associations are made in subroutine allok8_turb
!
      real(kind=rkd), pointer, save :: &
           cn(:), &  !  turbine nozzle coefficient.                             
         tinh(:), &  !  turbine inlet enthalpy.                           
         tinp(:), &  !  turbine inlet pressure.
         tinw(:)     !  turbine inlet flow.   
!
      integer(kind=ikd), target, allocatable, save :: i_turb(:)
      integer(kind=ikd), pointer, save :: &
         isgf(:), &  !  stage group subfile index.                        
        itsha(:), &  !  shaft number on which this turbine resides.       
        iturn(:), &  !  turbine number                                    
       jintur(:), &  !  retran junction of the turbine inlet.             
           ng(:)     !  number of stage groups in this turbine.           
!
!       
!  -------------------------------------------------------------------------------------------------------------------------------
!
!  Specifications for individual turbine stage group related arrays
!
      integer(kind=ikd), save :: & 
         num_tsg,  &  !  number of turbine stage groups
        siz_rtsg,  &  !  size of the array allocated for the real turb stage group pointer arrays
        siz_itsg      !  size of the array allocated for the integer turb stage group pointer arrays
!
      real(kind=rkd), target, allocatable, save :: r_tsg(:)
      real(kind=rkd), pointer, save :: &
          dth(:), &  !  enthalpy in stage group at design.                
          dtp(:), &  !  pressure in stage group at design.                
          dwp(:), &  !  flow in junction ijunsg at design.                
        ea2mb(:), &  !  governing stage 2nd moving blade exit angle.      
         eafb(:), &  !  governing stage fixed blade exit angle.           
         erat(:), &  !  stage efficiency ratio.                           
        g2mlc(:), &  !  governing stage 2nd moving blade loss coefficient.
          gar(:), &  !  group area ratio.                                 
         gblc(:), &  !  group blade loss coefficient.                     
          gbw(:), &  !  blade speed.                                      
          gea(:), &  !  group exit angle.                                 
         geta(:), &  !  group efficiency                                  
         gflc(:), &  !  governing stage fixed blade loss coefficient      
         gnlc(:), &  !  group nozzle angle.                               
         grlc(:), &  !  group reaction loss coefficient.                  
          gvo(:), &  !  ideal nozzle velocity.                            
          gxe(:), &  !  fraction of reaction for the governing stage.     
         lrem(:), &  !  residual stage losses.                            
        wdelh(:), &  !  power produced from the stage group.              
          xre(:)     !  fraction of reaction.                             
!         
      integer(kind=ikd), target, allocatable, save :: i_tsg(:)
      integer(kind=ikd), pointer, save :: &
         igov(:), &  !  governing stage flag.                             
       ijunsg(:), &  !  retran junction which links the previous stage group volume to ivolsg, or the turb inlet         
       ilturb(:), &  !  turbine loop index       
       ivolsg(:), &  !  retran volume number for this group.              
           ns(:)     !  number of stages represented by the group.        
!
      contains
!
!  ===============================================================================================================================
      subroutine allok8_turb_shafts  
!
!  Subroutine to allocate space and set pointers for turbine shaft related arrays
!     Note: num_shft must be defined before calling subroutine allok8_turbine_shafts.
!     allok8_turbine_shaft is called from subroutine inturb
! 
      use kind_specs
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_shft .le. 0) then
         write (6,110) num_shft
         call fail
         return
      endif
!
      if (allocated (r_shft)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_shft, i2,  shrpm, r_shft)
         call r_pntr (allocatd, num_shft, i2,  shpow, r_shft)
         call r_pntr (allocatd, num_shft, i2, powout, r_shft)
         call r_pntr (allocatd, num_shft, i2,  shtor, r_shft)
         call r_pntr (allocatd, num_shft, i2,   shir, r_shft)
         call r_pntr (allocatd, num_shft, i2, oshrpm, r_shft)
         call r_pntr (allocatd, num_shft, i2, shtori, r_shft)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rshft   = i2
!
            allocate (r_shft(siz_rshft), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rshft, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_shft
            block  = 'r_shft'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rshft, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_shft      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_shft, i2,  ntsha, i_shft)
         call i_pntr (allocatd, num_shft, i2,  nturs, i_shft)
         call i_pntr (allocatd, num_shft, i2,   isin, i_shft)
!
!     allocate space
!
         if (.not.allocatd) then
            siz_ishft   = i2
!
            allocate (i_shft(siz_ishft), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_ishft, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_shft
            block  = 'i_shft'
            call build_restart_block_list (block, rdum, idum, ldum, siz_ishft, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_shft      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' turbine shaft arrays (allok8_turbin_shafts).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for turbine shaft arrays (allok8_turbine_shafts).'/ &
          10x,'Number of turbine shafts = ',i6,', must be > 0 - programming error') 
      end subroutine allok8_turb_shafts
!
!  ===============================================================================================================================
      subroutine allok8_turbines
!
!  Subroutine to allocate space and set pointers for turbine related arrays
!      Note: num_turb must be defined before subroutine allok8_turbines is called
!     allok8_turbines is called from subroutine inturb 
! 
      use kind_specs
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (num_turb .le. 0) then
         write (6,110) num_turb
         call fail
         return
      endif
!
      if (allocated (r_turb)) return
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_turb, i2,     cn, r_turb)
         call r_pntr (allocatd, num_turb, i2,   tinh, r_turb)
         call r_pntr (allocatd, num_turb, i2,   tinp, r_turb)
         call r_pntr (allocatd, num_turb, i2,   tinw, r_turb)
!
!      allocate space
!
         if (.not.allocatd) then
            siz_rturb   = i2
!
            allocate (r_turb(siz_rturb), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rturb, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_turb
            block  = 'r_turb'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rturb, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_turb      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_turb, i2,   isgf, i_turb)
         call i_pntr (allocatd, num_turb, i2,  itsha, i_turb)
         call i_pntr (allocatd, num_turb, i2,  iturn, i_turb)         
         call i_pntr (allocatd, num_turb, i2, jintur, i_turb)
         call i_pntr (allocatd, num_turb, i2,     ng, i_turb)         
!
!     allocate space
!
         if (.not.allocatd) then
            siz_iturb   = i2
!
            allocate (i_turb(siz_iturb), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_iturb, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_turb
            block  = 'i_turb'
            call build_restart_block_list (block, rdum, idum, ldum, siz_iturb, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_turb      = 0
         endif
      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' turbine arrays (allok8_turbines).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for turbine arrays (allok8_turbines).'/ &
          10x,'Number of turbines = ',i6,', must be > 0 - programming error') 
!          
      end subroutine allok8_turbines
!
!  ===============================================================================================================================
      subroutine allok8_turb_stages
!
!  Subroutine to allocate space and set pointers for turbine related arrays
!     allok8_turbines is called from subroutine inturb 
! 
      use kind_specs
      use restart_block_list 
! 
!     Interface modules
!
      use s_r_pntr
      use s_i_pntr
      use s_l_pntr
! 
      implicit none
!
      character(kind=ckd,len=8) block
      integer(kind=ikd) l, i2, status
      logical(kind=lkd) allocatd
!
      real(kind=rkd),    pointer :: rdum(:)  !  dummy real pointer
      integer(kind=ikd), pointer :: idum(:)  !  dummy integer pointer
      logical(kind=lkd), pointer :: ldum(:)  !  dummy logical pointer
!
      if (allocated (r_tsg)) return
!
      if (.not.allocated (r_turb)) then
         write (6,110) 
         call fail
         return
      endif
!
!      num_tsg = 0
!
!      do l = 1, num_turb
!         num_tsg = num_tsg + ng(l)
!      enddo
!
      if (num_tsg .le. 0) then
         write (6,120) num_tsg
         call fail
         return
      endif
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for real arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call r_pntr (allocatd, num_tsg, i2,    dth, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,    dtp, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,    dwp, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,  ea2mb, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,   eafb, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,   erat, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,  g2mlc, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,    gar, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,   gblc, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,    gbw, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,    gea, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,   geta, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,   gflc, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,   gnlc, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,   grlc, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,    gvo, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,    gxe, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,   lrem, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,  wdelh, r_tsg)
         call r_pntr (allocatd, num_tsg, i2,    xre, r_tsg)
!         
!      allocate space
!
         if (.not.allocatd) then
            siz_rtsg   = i2
!
            allocate (r_tsg(siz_rtsg), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'real'
               write (6,100) block, siz_rtsg, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            rdum   => r_tsg
            block  = 'r_tsg'
            call build_restart_block_list (block, rdum, idum, ldum, siz_rtsg, 1, 2)
!
!     initialize real arrays to 0.0d0
!
         else
            r_tsg      = 0.0d0
         endif
      enddo
!
!  -------------------------------------------------------------------------------------------------------------------------------
!  Allocate space and assign pointers for integer arrays.  The first pass determines size of block to allocate and the second
!  pass assigns the pointer arrays
!  -------------------------------------------------------------------------------------------------------------------------------
!
      do l = 1, 2
         allocatd = l.eq.2
         i2       = 0
!
         call i_pntr (allocatd, num_tsg, i2,   igov, i_tsg)
         call i_pntr (allocatd, num_tsg, i2, ijunsg, i_tsg)
         call i_pntr (allocatd, num_tsg, i2, ilturb, i_tsg)
         call i_pntr (allocatd, num_tsg, i2, ivolsg, i_tsg)
         call i_pntr (allocatd, num_tsg, i2,     ns, i_tsg)
!         
!     allocate space
!
         if (.not.allocatd) then
            siz_itsg   = i2
!
            allocate (i_tsg(siz_itsg), stat=status)
!
!        check status of allocate - write error if allocate failed
!
            if (status .ne. 0) then
               block      = 'integer'
               write (6,100) block, siz_itsg, status
               call fail
            endif
!
!        Add block to list of blocks included in restart file
!
            idum   => i_tsg
            block  = 'i_tsg'
            call build_restart_block_list (block, rdum, idum, ldum, siz_itsg, 2, 2)
!
!     initialize integer arrays to 0
!
         else
            i_tsg      = 0
         endif
      enddo
!
!      i2        = 1
!
!  Initialize indices to first stage group for each turbine
!
!      do l = 1, num_turb
!         isgf(l)   = i2
!         i2        = i2 + ng(l)
!      enddo
!
      return
!
  100 format (' ******** Error allocating space for ',a8,' turbine stage group arrays (allok8_turbine_sgs).'/ &
          10x,'Length requested = ',i8/ &
          10x,'Status flag returned from allocate = ',i3)
  110 format (' ******** Error allocating space for turbine stage group arrays (allok8_turbine_sgs).'/ &
          10x,'Subroutine allok8_turbine_sgs called before allok8_turbines - programming error')
  120 format (' ******** Error allocating space for turbine stage group arrays (allok8_turbine_sgs).'/ &
          10x,'Sum of number of turbine stage groups for all turbines = ',i6,', must be > 0 - programming error') 
!          
      end subroutine allok8_turb_stages
!
      end module turbines
